@@ -6,6 +6,7 @@
  * \ingroup bke
  */
 
+#include <algorithm>
 #include <cstdio>
 #include <fcntl.h>
 #include <sys/stat.h>
@@ -18,30 +19,45 @@
 #include "MEM_guardedalloc.h"
 #include <cstring>
 
+#include "AS_essentials_library.hh"
+
 #include "DNA_ID.h"
 #include "DNA_image_types.h"
+#include "DNA_modifier_types.h"
 #include "DNA_packedFile_types.h"
 #include "DNA_sound_types.h"
 #include "DNA_vfont_types.h"
 #include "DNA_volume_types.h"
 
-#include "BLI_blenlib.h"
+#include "BLI_listbase.h"
+#include "BLI_path_utils.hh"
+#include "BLI_string.h"
 #include "BLI_utildefines.h"
 
-#include "BKE_image.h"
-#include "BKE_image_format.h"
-#include "BKE_main.h"
-#include "BKE_packedFile.h"
-#include "BKE_report.h"
+#include "BKE_bake_geometry_nodes_modifier.hh"
+#include "BKE_bake_geometry_nodes_modifier_pack.hh"
+#include "BKE_image.hh"
+#include "BKE_image_format.hh"
+#include "BKE_library.hh"
+#include "BKE_main.hh"
+#include "BKE_packedFile.hh"
+#include "BKE_report.hh"
 #include "BKE_sound.h"
-#include "BKE_vfont.h"
-#include "BKE_volume.h"
+#include "BKE_vfont.hh"
+#include "BKE_volume.hh"
 
-#include "IMB_imbuf.h"
-#include "IMB_imbuf_types.h"
+#include "DEG_depsgraph.hh"
+
+#include "IMB_imbuf.hh"
 
 #include "BLO_read_write.hh"
 
+#include "CLG_log.h"
+
+static CLG_LogRef LOG = {"bke.packedfile"};
+
+using namespace blender;
+
 int BKE_packedfile_seek(PackedFile *pf, int offset, int whence)
 {
   int oldseek = -1, seek = 0;
@@ -87,7 +103,7 @@ int BKE_packedfile_read(PackedFile *pf, void *data, int size)
     }
 
     if (size > 0) {
-      memcpy(data, ((char *)pf->data) + pf->seek, size);
+      memcpy(data, ((const char *)pf->data) + pf->seek, size);
     }
     else {
       size = 0;
@@ -102,42 +118,59 @@ int BKE_packedfile_read(PackedFile *pf, void *data, int size)
   return size;
 }
 
-int BKE_packedfile_count_all(Main *bmain)
+PackedFileCount BKE_packedfile_count_all(Main *bmain)
 {
   Image *ima;
   VFont *vf;
   bSound *sound;
   Volume *volume;
-  int count = 0;
+
+  PackedFileCount count;
 
   /* let's check if there are packed files... */
   for (ima = static_cast<Image *>(bmain->images.first); ima;
        ima = static_cast<Image *>(ima->id.next))
   {
-    if (BKE_image_has_packedfile(ima)) {
-      count++;
+    if (BKE_image_has_packedfile(ima) && !ID_IS_LINKED(ima)) {
+      count.individual_files++;
     }
   }
 
   for (vf = static_cast<VFont *>(bmain->fonts.first); vf; vf = static_cast<VFont *>(vf->id.next)) {
-    if (vf->packedfile) {
-      count++;
+    if (vf->packedfile && !ID_IS_LINKED(vf)) {
+      count.individual_files++;
     }
   }
 
   for (sound = static_cast<bSound *>(bmain->sounds.first); sound;
        sound = static_cast<bSound *>(sound->id.next))
   {
-    if (sound->packedfile) {
-      count++;
+    if (sound->packedfile && !ID_IS_LINKED(sound)) {
+      count.individual_files++;
     }
   }
 
   for (volume = static_cast<Volume *>(bmain->volumes.first); volume;
        volume = static_cast<Volume *>(volume->id.next))
   {
-    if (volume->packedfile) {
-      count++;
+    if (volume->packedfile && !ID_IS_LINKED(volume)) {
+      count.individual_files++;
+    }
+  }
+
+  LISTBASE_FOREACH (Object *, object, &bmain->objects) {
+    if (ID_IS_LINKED(object)) {
+      continue;
+    }
+    LISTBASE_FOREACH (ModifierData *, md, &object->modifiers) {
+      if (md->type == eModifierType_Nodes) {
+        NodesModifierData *nmd = reinterpret_cast<NodesModifierData *>(md);
+        for (const NodesModifierBake &bake : blender::Span{nmd->bakes, nmd->bakes_num}) {
+          if (bake.packed) {
+            count.bakes++;
+          }
+        }
+      }
     }
   }
 
@@ -148,8 +181,9 @@ void BKE_packedfile_free(PackedFile *pf)
 {
   if (pf) {
     BLI_assert(pf->data != nullptr);
+    BLI_assert(pf->sharing_info != nullptr);
 
-    MEM_SAFE_FREE(pf->data);
+    pf->sharing_info->remove_user_and_delete_if_last();
     MEM_freeN(pf);
   }
   else {
@@ -165,18 +199,25 @@ PackedFile *BKE_packedfile_duplicate(const PackedFile *pf_src)
   PackedFile *pf_dst;
 
   pf_dst = static_cast<PackedFile *>(MEM_dupallocN(pf_src));
-  pf_dst->data = MEM_dupallocN(pf_src->data);
+  pf_dst->sharing_info->add_user();
 
   return pf_dst;
 }
 
-PackedFile *BKE_packedfile_new_from_memory(void *mem, int memlen)
+PackedFile *BKE_packedfile_new_from_memory(const void *mem,
+                                           int memlen,
+                                           const blender::ImplicitSharingInfo *sharing_info)
 {
   BLI_assert(mem != nullptr);
+  if (!sharing_info) {
+    /* Assume we are the only owner of that memory currently. */
+    sharing_info = blender::implicit_sharing::info_for_mem_free(const_cast<void *>(mem));
+  }
 
-  PackedFile *pf = static_cast<PackedFile *>(MEM_callocN(sizeof(*pf), "PackedFile"));
+  PackedFile *pf = MEM_callocN<PackedFile>("PackedFile");
   pf->data = mem;
   pf->size = memlen;
+  pf->sharing_info = sharing_info;
 
   return pf;
 }
@@ -245,7 +286,7 @@ void BKE_packedfile_pack_all(Main *bmain, ReportList *reports, bool verbose)
   for (ima = static_cast<Image *>(bmain->images.first); ima;
        ima = static_cast<Image *>(ima->id.next))
   {
-    if (BKE_image_has_packedfile(ima) == false && !ID_IS_LINKED(ima)) {
+    if (BKE_image_has_packedfile(ima) == false && ID_IS_EDITABLE(ima)) {
       if (ELEM(ima->source, IMA_SRC_FILE, IMA_SRC_TILED)) {
         BKE_image_packfiles(reports, ima, ID_BLEND_PATH(bmain, &ima->id));
         tot++;
@@ -262,8 +303,9 @@ void BKE_packedfile_pack_all(Main *bmain, ReportList *reports, bool verbose)
   for (vfont = static_cast<VFont *>(bmain->fonts.first); vfont;
        vfont = static_cast<VFont *>(vfont->id.next))
   {
-    if (vfont->packedfile == nullptr && !ID_IS_LINKED(vfont) &&
-        BKE_vfont_is_builtin(vfont) == false) {
+    if (vfont->packedfile == nullptr && ID_IS_EDITABLE(vfont) &&
+        BKE_vfont_is_builtin(vfont) == false)
+    {
       vfont->packedfile = BKE_packedfile_new(
           reports, vfont->filepath, BKE_main_blendfile_path(bmain));
       tot++;
@@ -273,7 +315,7 @@ void BKE_packedfile_pack_all(Main *bmain, ReportList *reports, bool verbose)
   for (sound = static_cast<bSound *>(bmain->sounds.first); sound;
        sound = static_cast<bSound *>(sound->id.next))
   {
-    if (sound->packedfile == nullptr && !ID_IS_LINKED(sound)) {
+    if (sound->packedfile == nullptr && ID_IS_EDITABLE(sound)) {
       sound->packedfile = BKE_packedfile_new(
           reports, sound->filepath, BKE_main_blendfile_path(bmain));
       tot++;
@@ -283,13 +325,27 @@ void BKE_packedfile_pack_all(Main *bmain, ReportList *reports, bool verbose)
   for (volume = static_cast<Volume *>(bmain->volumes.first); volume;
        volume = static_cast<Volume *>(volume->id.next))
   {
-    if (volume->packedfile == nullptr && !ID_IS_LINKED(volume)) {
+    if (volume->packedfile == nullptr && ID_IS_EDITABLE(volume)) {
       volume->packedfile = BKE_packedfile_new(
           reports, volume->filepath, BKE_main_blendfile_path(bmain));
       tot++;
     }
   }
 
+  LISTBASE_FOREACH (Object *, object, &bmain->objects) {
+    if (ID_IS_LINKED(object)) {
+      continue;
+    }
+    LISTBASE_FOREACH (ModifierData *, md, &object->modifiers) {
+      if (md->type == eModifierType_Nodes) {
+        NodesModifierData *nmd = reinterpret_cast<NodesModifierData *>(md);
+        for (NodesModifierBake &bake : blender::MutableSpan{nmd->bakes, nmd->bakes_num}) {
+          blender::bke::bake::pack_geometry_nodes_bake(*bmain, reports, *object, *nmd, bake);
+        }
+      }
+    }
+  }
+
   if (tot > 0) {
     BKE_reportf(reports, RPT_INFO, "Packed %d file(s)", tot);
   }
@@ -366,7 +422,7 @@ int BKE_packedfile_write_to_file(ReportList *reports,
 
 enum ePF_FileCompare BKE_packedfile_compare_to_file(const char *ref_file_name,
                                                     const char *filepath_rel,
-                                                    PackedFile *pf)
+                                                    const PackedFile *pf)
 {
   BLI_stat_t st;
   enum ePF_FileCompare ret_val;
@@ -394,9 +450,7 @@ enum ePF_FileCompare BKE_packedfile_compare_to_file(const char *ref_file_name,
 
       for (int i = 0; i < pf->size; i += sizeof(buf)) {
         int len = pf->size - i;
-        if (len > sizeof(buf)) {
-          len = sizeof(buf);
-        }
+        len = std::min<ulong>(len, sizeof(buf));
 
         if (BLI_read(file, buf, len) != len) {
           /* read error ... */
@@ -513,7 +567,7 @@ static void unpack_generate_paths(const char *filepath,
       if (imapf != nullptr && imapf->packedfile != nullptr) {
         const PackedFile *pf = imapf->packedfile;
         enum eImbFileType ftype = eImbFileType(
-            IMB_ispic_type_from_memory((const uchar *)pf->data, pf->size));
+            IMB_test_image_type_from_memory((const uchar *)pf->data, pf->size));
         if (ima->source == IMA_SRC_TILED) {
           char tile_number[6];
           SNPRINTF(tile_number, ".%d", imapf->tile_number);
@@ -531,7 +585,7 @@ static void unpack_generate_paths(const char *filepath,
   }
 
   if (temp_dirname[0] == '\0') {
-    /* Fallback to relative dir. */
+    /* Fall back to relative dir. */
     STRNCPY(temp_dirname, "//");
   }
 
@@ -724,8 +778,8 @@ int BKE_packedfile_unpack_all_libraries(Main *bmain, ReportList *reports)
 
       newname = BKE_packedfile_unpack_to_file(reports,
                                               BKE_main_blendfile_path(bmain),
-                                              lib->filepath_abs,
-                                              lib->filepath_abs,
+                                              lib->runtime->filepath_abs,
+                                              lib->runtime->filepath_abs,
                                               lib->packedfile,
                                               PF_WRITE_ORIGINAL);
       if (newname != nullptr) {
@@ -748,10 +802,20 @@ void BKE_packedfile_pack_all_libraries(Main *bmain, ReportList *reports)
 {
   Library *lib;
 
-  /* Test for relativeness. */
+  /* Only allow libraries with relative paths (to avoid issues when unpacking, a limitation that we
+   * might want to lift since the "relativeness" does not really ensure sanity significantly more).
+   */
   for (lib = static_cast<Library *>(bmain->libraries.first); lib;
        lib = static_cast<Library *>(lib->id.next))
   {
+    /* Exception to the above: essential assets have an absolute path and should not prevent to
+     * operator from continuing. */
+    if (BLI_path_contains(blender::asset_system::essentials_directory_path().c_str(),
+                          lib->filepath))
+    {
+      continue;
+    }
+
     if (!BLI_path_is_rel(lib->filepath)) {
       break;
     }
@@ -765,6 +829,12 @@ void BKE_packedfile_pack_all_libraries(Main *bmain, ReportList *reports)
   for (lib = static_cast<Library *>(bmain->libraries.first); lib;
        lib = static_cast<Library *>(lib->id.next))
   {
+    /* Do not really pack essential assets though (see above). */
+    if (BLI_path_contains(blender::asset_system::essentials_directory_path().c_str(),
+                          lib->filepath))
+    {
+      continue;
+    }
     if (lib->packedfile == nullptr) {
       lib->packedfile = BKE_packedfile_new(reports, lib->filepath, BKE_main_blendfile_path(bmain));
     }
@@ -781,13 +851,13 @@ void BKE_packedfile_unpack_all(Main *bmain, ReportList *reports, enum ePF_FileSt
   for (ima = static_cast<Image *>(bmain->images.first); ima;
        ima = static_cast<Image *>(ima->id.next))
   {
-    if (BKE_image_has_packedfile(ima)) {
+    if (BKE_image_has_packedfile(ima) && !ID_IS_LINKED(ima)) {
       BKE_packedfile_unpack_image(bmain, reports, ima, how);
     }
   }
 
   for (vf = static_cast<VFont *>(bmain->fonts.first); vf; vf = static_cast<VFont *>(vf->id.next)) {
-    if (vf->packedfile) {
+    if (vf->packedfile && !ID_IS_LINKED(vf)) {
       BKE_packedfile_unpack_vfont(bmain, reports, vf, how);
     }
   }
@@ -795,7 +865,7 @@ void BKE_packedfile_unpack_all(Main *bmain, ReportList *reports, enum ePF_FileSt
   for (sound = static_cast<bSound *>(bmain->sounds.first); sound;
        sound = static_cast<bSound *>(sound->id.next))
   {
-    if (sound->packedfile) {
+    if (sound->packedfile && !ID_IS_LINKED(sound)) {
       BKE_packedfile_unpack_sound(bmain, reports, sound, how);
     }
   }
@@ -803,10 +873,25 @@ void BKE_packedfile_unpack_all(Main *bmain, ReportList *reports, enum ePF_FileSt
   for (volume = static_cast<Volume *>(bmain->volumes.first); volume;
        volume = static_cast<Volume *>(volume->id.next))
   {
-    if (volume->packedfile) {
+    if (volume->packedfile && !ID_IS_LINKED(volume)) {
       BKE_packedfile_unpack_volume(bmain, reports, volume, how);
     }
   }
+
+  LISTBASE_FOREACH (Object *, object, &bmain->objects) {
+    if (ID_IS_LINKED(object)) {
+      continue;
+    }
+    LISTBASE_FOREACH (ModifierData *, md, &object->modifiers) {
+      if (md->type == eModifierType_Nodes) {
+        NodesModifierData *nmd = reinterpret_cast<NodesModifierData *>(md);
+        for (NodesModifierBake &bake : blender::MutableSpan{nmd->bakes, nmd->bakes_num}) {
+          blender::bke::bake::unpack_geometry_nodes_bake(
+              *bmain, reports, *object, *nmd, bake, how);
+        }
+      }
+    }
+  }
 }
 
 bool BKE_packedfile_id_check(const ID *id)
@@ -840,6 +925,11 @@ bool BKE_packedfile_id_check(const ID *id)
 
 void BKE_packedfile_id_unpack(Main *bmain, ID *id, ReportList *reports, enum ePF_FileStatus how)
 {
+  /* Only unpack when datablock is editable. */
+  if (!ID_IS_EDITABLE(id)) {
+    return;
+  }
+
   switch (GS(id->name)) {
     case ID_IM: {
       Image *ima = (Image *)id;
@@ -879,28 +969,39 @@ void BKE_packedfile_id_unpack(Main *bmain, ID *id, ReportList *reports, enum ePF
   }
 }
 
-void BKE_packedfile_blend_write(BlendWriter *writer, PackedFile *pf)
+void BKE_packedfile_blend_write(BlendWriter *writer, const PackedFile *pf)
 {
   if (pf == nullptr) {
     return;
   }
   BLO_write_struct(writer, PackedFile, pf);
-  BLO_write_raw(writer, pf->size, pf->data);
+  BLO_write_shared(writer, pf->data, pf->size, pf->sharing_info, [&]() {
+    BLO_write_raw(writer, pf->size, pf->data);
+  });
 }
 
-void BKE_packedfile_blend_read(BlendDataReader *reader, PackedFile **pf_p)
+void BKE_packedfile_blend_read(BlendDataReader *reader, PackedFile **pf_p, StringRefNull filepath)
 {
-  BLO_read_packed_address(reader, pf_p);
+  BLO_read_struct(reader, PackedFile, pf_p);
   PackedFile *pf = *pf_p;
   if (pf == nullptr) {
     return;
   }
-
-  BLO_read_packed_address(reader, &pf->data);
+  /* NOTE: there is no way to handle endianness switch here. */
+  pf->sharing_info = BLO_read_shared(reader, &pf->data, [&]() {
+    BLO_read_data_address(reader, &pf->data);
+    /* Do not create an implicit sharing if read data pointer is `nullptr`. */
+    return pf->data ? blender::implicit_sharing::info_for_mem_free(const_cast<void *>(pf->data)) :
+                      nullptr;
+  });
   if (pf->data == nullptr) {
-    /* We cannot allow a PackedFile with a nullptr data field,
+    /* We cannot allow a #PackedFile with a nullptr data field,
      * the whole code assumes this is not possible. See #70315. */
-    printf("%s: nullptr packedfile data, cleaning up...\n", __func__);
-    MEM_SAFE_FREE(pf);
+    CLOG_WARN(&LOG,
+              "%s: nullptr packedfile data (source: '%s'), cleaning up...",
+              __func__,
+              filepath.c_str());
+    BLI_assert(pf->sharing_info == nullptr);
+    MEM_SAFE_FREE(*pf_p);
   }
 }
