@@ -8,10 +8,6 @@
  * \ingroup bli
  */
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 /**
  * \note keep \a sort_value first,
  * so cmp functions can be reused.
@@ -41,7 +37,3 @@ int BLI_sortutil_cmp_float_reverse(const void *a_, const void *b_);
 
 int BLI_sortutil_cmp_int(const void *a_, const void *b_);
 int BLI_sortutil_cmp_int_reverse(const void *a_, const void *b_);
-
-#ifdef __cplusplus
-}
-#endif
