@@ -13,7 +13,7 @@
 /* avoid many includes for now */
 #include "BLI_compiler_compat.h"
 #include "BLI_sys_types.h"
-#include "BLI_utildefines_variadic.h"
+#include "BLI_utildefines_variadic.h"  // IWYU prama: export
 
 /* We could remove in future. */
 #include "BLI_assert.h"
@@ -32,85 +32,11 @@ extern "C" {
 /** \name Min/Max Macros
  * \{ */
 
-/* useful for finding bad use of min/max */
-#if 0
-/* gcc only */
-#  define _TYPECHECK(a, b) ((void)(((typeof(a) *)0) == ((typeof(b) *)0)))
-#  define MIN2(x, y) (_TYPECHECK(x, y), (((x) < (y) ? (x) : (y))))
-#  define MAX2(x, y) (_TYPECHECK(x, y), (((x) > (y) ? (x) : (y))))
-#endif
-
-/* min/max */
-#if defined(__GNUC__) || defined(__clang__)
-
-#  define MIN2(a, b) \
-    __extension__({ \
-      typeof(a) a_ = (a); \
-      typeof(b) b_ = (b); \
-      ((a_) < (b_) ? (a_) : (b_)); \
-    })
-
-#  define MAX2(a, b) \
-    __extension__({ \
-      typeof(a) a_ = (a); \
-      typeof(b) b_ = (b); \
-      ((a_) > (b_) ? (a_) : (b_)); \
-    })
-
-#  define MIN3(a, b, c) \
-    __extension__({ \
-      typeof(a) a_ = (a); \
-      typeof(b) b_ = (b); \
-      typeof(c) c_ = (c); \
-      ((a_ < b_) ? ((a_ < c_) ? a_ : c_) : ((b_ < c_) ? b_ : c_)); \
-    })
-
-#  define MAX3(a, b, c) \
-    __extension__({ \
-      typeof(a) a_ = (a); \
-      typeof(b) b_ = (b); \
-      typeof(c) c_ = (c); \
-      ((a_ > b_) ? ((a_ > c_) ? a_ : c_) : ((b_ > c_) ? b_ : c_)); \
-    })
-
-#  define MIN4(a, b, c, d) \
-    __extension__({ \
-      typeof(a) a_ = (a); \
-      typeof(b) b_ = (b); \
-      typeof(c) c_ = (c); \
-      typeof(d) d_ = (d); \
-      ((a_ < b_) ? ((a_ < c_) ? ((a_ < d_) ? a_ : d_) : ((c_ < d_) ? c_ : d_)) : \
-                   ((b_ < c_) ? ((b_ < d_) ? b_ : d_) : ((c_ < d_) ? c_ : d_))); \
-    })
-
-#  define MAX4(a, b, c, d) \
-    __extension__({ \
-      typeof(a) a_ = (a); \
-      typeof(b) b_ = (b); \
-      typeof(c) c_ = (c); \
-      typeof(d) d_ = (d); \
-      ((a_ > b_) ? ((a_ > c_) ? ((a_ > d_) ? a_ : d_) : ((c_ > d_) ? c_ : d_)) : \
-                   ((b_ > c_) ? ((b_ > d_) ? b_ : d_) : ((c_ > d_) ? c_ : d_))); \
-    })
-
-#else
+#ifndef __cplusplus
 #  define MIN2(a, b) ((a) < (b) ? (a) : (b))
 #  define MAX2(a, b) ((a) > (b) ? (a) : (b))
-
-#  define MIN3(a, b, c) (MIN2(MIN2((a), (b)), (c)))
-#  define MIN4(a, b, c, d) (MIN2(MIN2((a), (b)), MIN2((c), (d))))
-
-#  define MAX3(a, b, c) (MAX2(MAX2((a), (b)), (c)))
-#  define MAX4(a, b, c, d) (MAX2(MAX2((a), (b)), MAX2((c), (d))))
 #endif
 
-/* min/max that return a value of our choice */
-#define MAX3_PAIR(cmp_a, cmp_b, cmp_c, ret_a, ret_b, ret_c) \
-  ((cmp_a > cmp_b) ? ((cmp_a > cmp_c) ? ret_a : ret_c) : ((cmp_b > cmp_c) ? ret_b : ret_c))
-
-#define MIN3_PAIR(cmp_a, cmp_b, cmp_c, ret_a, ret_b, ret_c) \
-  ((cmp_a < cmp_b) ? ((cmp_a < cmp_c) ? ret_a : ret_c) : ((cmp_b < cmp_c) ? ret_b : ret_c))
-
 #define INIT_MINMAX(min, max) \
   { \
     (min)[0] = (min)[1] = (min)[2] = 1.0e30f; \
@@ -123,70 +49,6 @@ extern "C" {
     (max)[0] = (max)[1] = -1.0e30f; \
   } \
   (void)0
-#define DO_MIN(vec, min) \
-  { \
-    if ((min)[0] > (vec)[0]) { \
-      (min)[0] = (vec)[0]; \
-    } \
-    if ((min)[1] > (vec)[1]) { \
-      (min)[1] = (vec)[1]; \
-    } \
-    if ((min)[2] > (vec)[2]) { \
-      (min)[2] = (vec)[2]; \
-    } \
-  } \
-  (void)0
-#define DO_MAX(vec, max) \
-  { \
-    if ((max)[0] < (vec)[0]) { \
-      (max)[0] = (vec)[0]; \
-    } \
-    if ((max)[1] < (vec)[1]) { \
-      (max)[1] = (vec)[1]; \
-    } \
-    if ((max)[2] < (vec)[2]) { \
-      (max)[2] = (vec)[2]; \
-    } \
-  } \
-  (void)0
-#define DO_MINMAX(vec, min, max) \
-  { \
-    if ((min)[0] > (vec)[0]) { \
-      (min)[0] = (vec)[0]; \
-    } \
-    if ((min)[1] > (vec)[1]) { \
-      (min)[1] = (vec)[1]; \
-    } \
-    if ((min)[2] > (vec)[2]) { \
-      (min)[2] = (vec)[2]; \
-    } \
-    if ((max)[0] < (vec)[0]) { \
-      (max)[0] = (vec)[0]; \
-    } \
-    if ((max)[1] < (vec)[1]) { \
-      (max)[1] = (vec)[1]; \
-    } \
-    if ((max)[2] < (vec)[2]) { \
-      (max)[2] = (vec)[2]; \
-    } \
-  } \
-  (void)0
-#define DO_MINMAX2(vec, min, max) \
-  { \
-    if ((min)[0] > (vec)[0]) { \
-      (min)[0] = (vec)[0]; \
-    } \
-    if ((min)[1] > (vec)[1]) { \
-      (min)[1] = (vec)[1]; \
-    } \
-    if ((max)[0] < (vec)[0]) { \
-      (max)[0] = (vec)[0]; \
-    } \
-    if ((max)[1] < (vec)[1]) { \
-      (max)[1] = (vec)[1]; \
-    } \
-  } \
-  (void)0
 
 /** \} */
 
@@ -205,17 +67,6 @@ extern "C" {
   } \
   (void)0
 
-/* swap with a temp value */
-#define SWAP_TVAL(tval, a, b) \
-  { \
-    CHECK_TYPE_PAIR(tval, a); \
-    CHECK_TYPE_PAIR(tval, b); \
-    (tval) = (a); \
-    (a) = (b); \
-    (b) = (tval); \
-  } \
-  (void)0
-
 /* shift around elements */
 #define SHIFT3(type, a, b, c) \
   { \
@@ -325,14 +176,38 @@ extern "C" {
  */
 #define DECIMAL_DIGITS_BOUND(t) (241 * sizeof(t) / 100 + 1)
 
+#ifdef __cplusplus
+inline constexpr int64_t is_power_of_2(const int64_t x)
+{
+  BLI_assert(x >= 0);
+  return (x & (x - 1)) == 0;
+}
+
+inline constexpr int64_t log2_floor(const int64_t x)
+{
+  BLI_assert(x >= 0);
+  return x <= 1 ? 0 : 1 + log2_floor(x >> 1);
+}
+
+inline constexpr int64_t log2_ceil(const int64_t x)
+{
+  BLI_assert(x >= 0);
+  return (is_power_of_2(int(x))) ? log2_floor(x) : log2_floor(x) + 1;
+}
+
+inline constexpr int64_t power_of_2_max(const int64_t x)
+{
+  BLI_assert(x >= 0);
+  return 1ll << log2_ceil(x);
+}
+#endif
+
 /** \} */
 
 /* -------------------------------------------------------------------- */
 /** \name Clamp Macros
  * \{ */
 
-#define CLAMPIS(a, b, c) ((a) < (b) ? (b) : (a) > (c) ? (c) : (a))
-
 #define CLAMP(a, b, c) \
   { \
     if ((a) < (b)) { \
@@ -360,78 +235,6 @@ extern "C" {
   } \
   (void)0
 
-#define CLAMP2(vec, b, c) \
-  { \
-    CLAMP((vec)[0], b, c); \
-    CLAMP((vec)[1], b, c); \
-  } \
-  (void)0
-
-#define CLAMP2_MIN(vec, b) \
-  { \
-    CLAMP_MIN((vec)[0], b); \
-    CLAMP_MIN((vec)[1], b); \
-  } \
-  (void)0
-
-#define CLAMP2_MAX(vec, b) \
-  { \
-    CLAMP_MAX((vec)[0], b); \
-    CLAMP_MAX((vec)[1], b); \
-  } \
-  (void)0
-
-#define CLAMP3(vec, b, c) \
-  { \
-    CLAMP((vec)[0], b, c); \
-    CLAMP((vec)[1], b, c); \
-    CLAMP((vec)[2], b, c); \
-  } \
-  (void)0
-
-#define CLAMP3_MIN(vec, b) \
-  { \
-    CLAMP_MIN((vec)[0], b); \
-    CLAMP_MIN((vec)[1], b); \
-    CLAMP_MIN((vec)[2], b); \
-  } \
-  (void)0
-
-#define CLAMP3_MAX(vec, b) \
-  { \
-    CLAMP_MAX((vec)[0], b); \
-    CLAMP_MAX((vec)[1], b); \
-    CLAMP_MAX((vec)[2], b); \
-  } \
-  (void)0
-
-#define CLAMP4(vec, b, c) \
-  { \
-    CLAMP((vec)[0], b, c); \
-    CLAMP((vec)[1], b, c); \
-    CLAMP((vec)[2], b, c); \
-    CLAMP((vec)[3], b, c); \
-  } \
-  (void)0
-
-#define CLAMP4_MIN(vec, b) \
-  { \
-    CLAMP_MIN((vec)[0], b); \
-    CLAMP_MIN((vec)[1], b); \
-    CLAMP_MIN((vec)[2], b); \
-    CLAMP_MIN((vec)[3], b); \
-  } \
-  (void)0
-
-#define CLAMP4_MAX(vec, b) \
-  { \
-    CLAMP_MAX((vec)[0], b); \
-    CLAMP_MAX((vec)[1], b); \
-    CLAMP_MAX((vec)[2], b); \
-    CLAMP_MAX((vec)[3], b); \
-  } \
-  (void)0
-
 /** \} */
 
 /* -------------------------------------------------------------------- */
@@ -453,48 +256,9 @@ extern "C" {
 /** \name Array Macros
  * \{ */
 
-/* array helpers */
-#define ARRAY_LAST_ITEM(arr_start, arr_dtype, arr_len) \
-  (arr_dtype *)((char *)(arr_start) + (sizeof(*((arr_dtype *)NULL)) * (size_t)(arr_len - 1)))
-
 #define ARRAY_HAS_ITEM(arr_item, arr_start, arr_len) \
   (CHECK_TYPE_PAIR_INLINE(arr_start, arr_item), \
-   ((unsigned int)((arr_item) - (arr_start)) < (unsigned int)(arr_len)))
-
-/**
- * \note use faster #ARRAY_DELETE_REORDER_LAST when we can re-order.
- */
-#define ARRAY_DELETE(arr, index, delete_len, arr_len) \
-  { \
-    BLI_assert((&arr[index] >= arr) && ((index) + delete_len <= arr_len)); \
-    memmove(&(arr)[index], \
-            &(arr)[(index) + (delete_len)], \
-            (((arr_len) - (index)) - (delete_len)) * sizeof(*(arr))); \
-  } \
-  ((void)0)
-
-/**
- * Re-ordering array removal.
- *
- * When removing single items this compiles down to:
- * `if (index + 1 != arr_len) { arr[index] = arr[arr_len - 1]; }` (typical reordering removal),
- * with removing multiple items, overlap is detected to avoid memcpy errors.
- */
-#define ARRAY_DELETE_REORDER_LAST(arr, index, delete_len, arr_len) \
-  { \
-    BLI_assert((&arr[index] >= arr) && ((index) + delete_len <= arr_len)); \
-    if ((index) + (delete_len) != (arr_len)) { \
-      if (((delete_len) == 1) || ((delete_len) <= ((arr_len) - ((index) + (delete_len))))) { \
-        memcpy(&(arr)[index], &(arr)[(arr_len) - (delete_len)], (delete_len) * sizeof(*(arr))); \
-      } \
-      else { \
-        memcpy(&(arr)[index], \
-               &(arr)[(arr_len) - ((arr_len) - ((index) + (delete_len)))], \
-               ((arr_len) - ((index) + (delete_len))) * sizeof(*(arr))); \
-      } \
-    } \
-  } \
-  ((void)0)
+   ((size_t)((arr_item) - (arr_start)) < (size_t)(arr_len)))
 
 /* assuming a static array */
 #ifndef __cplusplus
@@ -570,11 +334,8 @@ extern "C" {
 /** \name Pointer Macros
  * \{ */
 
-#if defined(__GNUC__) || defined(__clang__)
-#  define POINTER_OFFSET(v, ofs) ((typeof(v))((char *)(v) + (ofs)))
-#else
-#  define POINTER_OFFSET(v, ofs) ((void *)((char *)(v) + (ofs)))
-#endif
+#define POINTER_OFFSET(v, ofs) \
+  (reinterpret_cast<typename std::remove_reference<decltype(v)>::type>((char *)(v) + (ofs)))
 
 /* Warning-free macros for storing ints in pointers. Use these _only_
  * for storing an int in a pointer, not a pointer in an int (64bit)! */
@@ -594,7 +355,7 @@ extern "C" {
  *
  * \{ */
 
-/** Performs `offsetof(typeof(data), member) + sizeof((data)->member)` for non-gcc compilers. */
+/** Performs `offsetof(decltype(data), member) + sizeof((data)->member)` for non-gcc compilers. */
 #define OFFSETOF_STRUCT_AFTER(_struct, _member) \
   ((size_t)(((const char *)&((_struct)->_member)) - ((const char *)(_struct))) + \
    sizeof((_struct)->_member))
@@ -827,16 +588,6 @@ extern bool BLI_memory_is_zero(const void *arr, size_t arr_size);
 #  define ENUM_OPERATORS(_type, _max)
 #endif
 
-/**
- * Utility so function declarations in C headers can use C++ default arguments. The default is then
- * available when included in a C++ file, otherwise the argument has to be set explicitly.
- */
-#ifdef __cplusplus
-#  define CPP_ARG_DEFAULT(default_value) = default_value
-#else
-#  define CPP_ARG_DEFAULT(default_value)
-#endif
-
 /** \} */
 
 /* -------------------------------------------------------------------- */
@@ -858,7 +609,7 @@ extern bool BLI_memory_is_zero(const void *arr, size_t arr_size);
  */
 #define BLI_ENABLE_IF(condition) typename std::enable_if_t<(condition)> * = nullptr
 
-#if defined(_MSC_VER)
+#if defined(_MSC_VER) && !defined(__clang__)
 #  define BLI_NO_UNIQUE_ADDRESS [[msvc::no_unique_address]]
 #elif defined(__has_cpp_attribute)
 #  if __has_cpp_attribute(no_unique_address)
@@ -878,10 +629,8 @@ extern bool BLI_memory_is_zero(const void *arr, size_t arr_size);
 namespace blender::blenlib_internal {
 
 /* A replacement for std::is_bounded_array_v until we go C++20. */
-template<class T> struct IsBoundedArray : std::false_type {
-};
-template<class T, std::size_t N> struct IsBoundedArray<T[N]> : std::true_type {
-};
+template<class T> struct IsBoundedArray : std::false_type {};
+template<class T, std::size_t N> struct IsBoundedArray<T[N]> : std::true_type {};
 
 }  // namespace blender::blenlib_internal
 
