@@ -88,7 +88,8 @@ static bool apply_color_operation_for_mode(const VertexColorMode mode,
   return changed;
 }
 
-static int grease_pencil_vertex_paint_brightness_contrast_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus grease_pencil_vertex_paint_brightness_contrast_exec(bContext *C,
+                                                                            wmOperator *op)
 {
   const Scene &scene = *CTX_data_scene(C);
   Object &object = *CTX_data_active_object(C);
@@ -150,7 +151,7 @@ static void GREASE_PENCIL_OT_vertex_color_brightness_contrast(wmOperatorType *ot
   ot->idname = "GREASE_PENCIL_OT_vertex_color_brightness_contrast";
   ot->description = "Adjust vertex color brightness/contrast";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = grease_pencil_vertex_paint_brightness_contrast_exec;
   ot->poll = grease_pencil_vertex_painting_poll;
 
@@ -165,7 +166,7 @@ static void GREASE_PENCIL_OT_vertex_color_brightness_contrast(wmOperatorType *ot
   RNA_def_float(ot->srna, "contrast", 0.0f, -1.0f, 1.0f, "Contrast", "", -1.0f, 1.0f);
 }
 
-static int grease_pencil_vertex_paint_hsv_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus grease_pencil_vertex_paint_hsv_exec(bContext *C, wmOperator *op)
 {
   const Scene &scene = *CTX_data_scene(C);
   Object &object = *CTX_data_active_object(C);
@@ -224,7 +225,7 @@ static void GREASE_PENCIL_OT_vertex_color_hsv(wmOperatorType *ot)
   ot->idname = "GREASE_PENCIL_OT_vertex_color_hsv";
   ot->description = "Adjust vertex color HSV values";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = grease_pencil_vertex_paint_hsv_exec;
   ot->poll = grease_pencil_vertex_painting_poll;
 
@@ -239,7 +240,7 @@ static void GREASE_PENCIL_OT_vertex_color_hsv(wmOperatorType *ot)
   RNA_def_float(ot->srna, "v", 1.0f, 0.0f, 2.0f, "Value", "", 0.0f, 2.0f);
 }
 
-static int grease_pencil_vertex_paint_invert_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus grease_pencil_vertex_paint_invert_exec(bContext *C, wmOperator *op)
 {
   const Scene &scene = *CTX_data_scene(C);
   Object &object = *CTX_data_active_object(C);
@@ -281,7 +282,7 @@ static void GREASE_PENCIL_OT_vertex_color_invert(wmOperatorType *ot)
   ot->idname = "GREASE_PENCIL_OT_vertex_color_invert";
   ot->description = "Invert RGB values";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = grease_pencil_vertex_paint_invert_exec;
   ot->poll = grease_pencil_vertex_painting_poll;
 
@@ -293,7 +294,7 @@ static void GREASE_PENCIL_OT_vertex_color_invert(wmOperatorType *ot)
       ot->srna, "mode", prop_grease_pencil_vertex_mode, int(VertexColorMode::Both), "Mode", "");
 }
 
-static int grease_pencil_vertex_paint_levels_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus grease_pencil_vertex_paint_levels_exec(bContext *C, wmOperator *op)
 {
   const Scene &scene = *CTX_data_scene(C);
   Object &object = *CTX_data_active_object(C);
@@ -337,7 +338,7 @@ static void GREASE_PENCIL_OT_vertex_color_levels(wmOperatorType *ot)
   ot->idname = "GREASE_PENCIL_OT_vertex_color_levels";
   ot->description = "Adjust levels of vertex colors";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = grease_pencil_vertex_paint_levels_exec;
   ot->poll = grease_pencil_vertex_painting_poll;
 
@@ -354,7 +355,7 @@ static void GREASE_PENCIL_OT_vertex_color_levels(wmOperatorType *ot)
       ot->srna, "gain", 1.0f, 0.0f, FLT_MAX, "Gain", "Value to multiply colors by", 0.0f, 10.0f);
 }
 
-static int grease_pencil_vertex_paint_set_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus grease_pencil_vertex_paint_set_exec(bContext *C, wmOperator *op)
 {
   const Scene &scene = *CTX_data_scene(C);
   Object &object = *CTX_data_active_object(C);
@@ -412,7 +413,7 @@ static void GREASE_PENCIL_OT_vertex_color_set(wmOperatorType *ot)
   ot->idname = "GREASE_PENCIL_OT_vertex_color_set";
   ot->description = "Set active color to all selected vertex";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = grease_pencil_vertex_paint_set_exec;
   ot->poll = grease_pencil_vertex_painting_poll;
 
@@ -425,7 +426,7 @@ static void GREASE_PENCIL_OT_vertex_color_set(wmOperatorType *ot)
   RNA_def_float(ot->srna, "factor", 1.0f, 0.0f, 1.0f, "Factor", "Mix Factor", 0.0f, 1.0f);
 }
 
-static int grease_pencil_vertex_paint_reset_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus grease_pencil_vertex_paint_reset_exec(bContext *C, wmOperator *op)
 {
   const Scene &scene = *CTX_data_scene(C);
   Object &object = *CTX_data_active_object(C);
