@@ -6,10 +6,12 @@
  * \ingroup edinterface
  */
 
+#include <algorithm>
 #include <cctype>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
+#include <fmt/format.h>
 
 #include "DNA_object_types.h"
 #include "DNA_screen_types.h"
@@ -20,18 +22,18 @@
 #include "BLI_string.h"
 #include "BLI_utildefines.h"
 
-#include "BLT_translation.h"
+#include "BLT_translation.hh"
 
-#include "BKE_context.h"
-#include "BKE_global.h"
-#include "BKE_idprop.h"
-#include "BKE_lib_id.h"
-#include "BKE_report.h"
+#include "BKE_context.hh"
+#include "BKE_global.hh"
+#include "BKE_idprop.hh"
+#include "BKE_lib_id.hh"
 #include "BKE_screen.hh"
 
 #include "MEM_guardedalloc.h"
 
 #include "RNA_access.hh"
+#include "RNA_prototypes.hh"
 
 #include "UI_interface.hh"
 #include "UI_interface_icons.hh"
@@ -44,13 +46,16 @@
 
 #include "interface_intern.hh"
 
+using blender::StringRef;
+using blender::StringRefNull;
+
 /*************************** RNA Utilities ******************************/
 
 uiBut *uiDefAutoButR(uiBlock *block,
                      PointerRNA *ptr,
                      PropertyRNA *prop,
                      int index,
-                     const char *name,
+                     const std::optional<StringRef> name,
                      int icon,
                      int x,
                      int y,
@@ -65,7 +70,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
         return nullptr;
       }
 
-      if (icon && name && name[0] == '\0') {
+      if (icon && name && name->is_empty()) {
         but = uiDefIconButR_prop(block,
                                  UI_BTYPE_ICON_TOGGLE,
                                  0,
@@ -79,9 +84,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
                                  index,
                                  0,
                                  0,
-                                 -1,
-                                 -1,
-                                 nullptr);
+                                 std::nullopt);
       }
       else if (icon) {
         but = uiDefIconTextButR_prop(block,
@@ -98,9 +101,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
                                      index,
                                      0,
                                      0,
-                                     -1,
-                                     -1,
-                                     nullptr);
+                                     std::nullopt);
       }
       else {
         but = uiDefButR_prop(block,
@@ -116,9 +117,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
                              index,
                              0,
                              0,
-                             -1,
-                             -1,
-                             nullptr);
+                             std::nullopt);
       }
       break;
     }
@@ -139,9 +138,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
                                -1,
                                0,
                                0,
-                               0,
-                               0,
-                               nullptr);
+                               std::nullopt);
         }
         else {
           return nullptr;
@@ -163,9 +160,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
                              index,
                              0,
                              0,
-                             -1,
-                             -1,
-                             nullptr);
+                             std::nullopt);
       }
       else {
         but = uiDefButR_prop(block,
@@ -181,9 +176,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
                              index,
                              0,
                              0,
-                             0,
-                             0,
-                             nullptr);
+                             std::nullopt);
       }
 
       if (RNA_property_flag(prop) & PROP_TEXTEDIT_UPDATE) {
@@ -192,7 +185,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
       break;
     }
     case PROP_ENUM:
-      if (icon && name && name[0] == '\0') {
+      if (icon && name && name->is_empty()) {
         but = uiDefIconButR_prop(block,
                                  UI_BTYPE_MENU,
                                  0,
@@ -206,16 +199,14 @@ uiBut *uiDefAutoButR(uiBlock *block,
                                  index,
                                  0,
                                  0,
-                                 -1,
-                                 -1,
-                                 nullptr);
+                                 std::nullopt);
       }
       else if (icon) {
         but = uiDefIconTextButR_prop(block,
                                      UI_BTYPE_MENU,
                                      0,
                                      icon,
-                                     nullptr,
+                                     std::nullopt,
                                      x,
                                      y,
                                      width,
@@ -225,9 +216,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
                                      index,
                                      0,
                                      0,
-                                     -1,
-                                     -1,
-                                     nullptr);
+                                     std::nullopt);
       }
       else {
         but = uiDefButR_prop(block,
@@ -243,13 +232,11 @@ uiBut *uiDefAutoButR(uiBlock *block,
                              index,
                              0,
                              0,
-                             -1,
-                             -1,
-                             nullptr);
+                             std::nullopt);
       }
       break;
     case PROP_STRING:
-      if (icon && name && name[0] == '\0') {
+      if (icon && name && name->is_empty()) {
         but = uiDefIconButR_prop(block,
                                  UI_BTYPE_TEXT,
                                  0,
@@ -263,9 +250,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
                                  index,
                                  0,
                                  0,
-                                 -1,
-                                 -1,
-                                 nullptr);
+                                 std::nullopt);
       }
       else if (icon) {
         but = uiDefIconTextButR_prop(block,
@@ -282,9 +267,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
                                      index,
                                      0,
                                      0,
-                                     -1,
-                                     -1,
-                                     nullptr);
+                                     std::nullopt);
       }
       else {
         but = uiDefButR_prop(block,
@@ -300,9 +283,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
                              index,
                              0,
                              0,
-                             -1,
-                             -1,
-                             nullptr);
+                             std::nullopt);
       }
 
       if (RNA_property_flag(prop) & PROP_TEXTEDIT_UPDATE) {
@@ -334,9 +315,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
                                    index,
                                    0,
                                    0,
-                                   -1,
-                                   -1,
-                                   nullptr);
+                                   std::nullopt);
       ui_but_add_search(but, ptr, prop, nullptr, nullptr, false);
       break;
     }
@@ -344,7 +323,7 @@ uiBut *uiDefAutoButR(uiBlock *block,
       char text[256];
       SNPRINTF(text, IFACE_("%d items"), RNA_property_collection_length(ptr, prop));
       but = uiDefBut(
-          block, UI_BTYPE_LABEL, 0, text, x, y, width, height, nullptr, 0, 0, 0, 0, nullptr);
+          block, UI_BTYPE_LABEL, 0, text, x, y, width, height, nullptr, 0, 0, std::nullopt);
       UI_but_flag_enable(but, UI_BUT_DISABLED);
       break;
     }
@@ -391,7 +370,7 @@ eAutoPropButsReturn uiDefAutoButsRNA(uiLayout *layout,
 {
   eAutoPropButsReturn return_info = UI_PROP_BUTS_NONE_ADDED;
   uiLayout *col;
-  const char *name;
+  std::optional<StringRefNull> name;
 
   RNA_STRUCT_BEGIN (ptr, prop) {
     const int flag = RNA_property_flag(prop);
@@ -413,16 +392,16 @@ eAutoPropButsReturn uiDefAutoButsRNA(uiLayout *layout,
         name = RNA_property_ui_name(prop);
 
         if (label_align == UI_BUT_LABEL_ALIGN_COLUMN) {
-          col = uiLayoutColumn(layout, true);
+          col = &layout->column(true);
 
           if (!is_boolean) {
-            uiItemL(col, name, ICON_NONE);
+            col->label(*name, ICON_NONE);
           }
         }
         else {
           BLI_assert(label_align == UI_BUT_LABEL_ALIGN_SPLIT_COLUMN);
-          col = uiLayoutColumn(layout, true);
-          /* Let uiItemFullR() create the split layout. */
+          col = &layout->column(true);
+          /* Let uiLayout::prop() create the split layout. */
           uiLayoutSetPropSep(col, true);
         }
 
@@ -431,7 +410,7 @@ eAutoPropButsReturn uiDefAutoButsRNA(uiLayout *layout,
       case UI_BUT_LABEL_ALIGN_NONE:
       default:
         col = layout;
-        name = nullptr; /* no smart label alignment, show default name with button */
+        name = std::nullopt; /* no smart label alignment, show default name with button */
         break;
     }
 
@@ -443,8 +422,7 @@ eAutoPropButsReturn uiDefAutoButsRNA(uiLayout *layout,
       uiLayoutSetActivateInit(col, true);
     }
 
-    uiItemFullR(
-        col, ptr, prop, -1, 0, compact ? UI_ITEM_R_COMPACT : UI_ITEM_NONE, name, ICON_NONE);
+    col->prop(ptr, prop, -1, 0, compact ? UI_ITEM_R_COMPACT : UI_ITEM_NONE, name, ICON_NONE);
     return_info &= ~UI_PROP_BUTS_NONE_ADDED;
 
     if (use_activate_init) {
@@ -464,51 +442,46 @@ void UI_but_func_identity_compare_set(uiBut *but, uiButIdentityCompareFunc cmp_f
 /* *** RNA collection search menu *** */
 
 struct CollItemSearch {
-  CollItemSearch *next, *prev;
   void *data;
-  char *name;
+  std::string name;
   int index;
   int iconid;
-  bool is_id;
   int name_prefix_offset;
-  uint has_sep_char : 1;
+  bool is_id;
+  bool has_sep_char;
 };
 
-static bool add_collection_search_item(CollItemSearch *cis,
+static bool add_collection_search_item(CollItemSearch &cis,
                                        const bool requires_exact_data_name,
                                        const bool has_id_icon,
                                        uiSearchItems *items)
 {
-  char name_buf[UI_MAX_DRAW_STR];
 
-  /* If no item has an own icon to display, libraries can use the library icons rather than the
+  /* If no item has its own icon to display, libraries can use the library icons rather than the
    * name prefix for showing the library status. */
-  int name_prefix_offset = cis->name_prefix_offset;
-  if (!has_id_icon && cis->is_id && !requires_exact_data_name) {
-    cis->iconid = UI_icon_from_library(static_cast<const ID *>(cis->data));
-    /* No need to re-allocate, string should be shorter than before (lib status prefix is
-     * removed). */
+  int name_prefix_offset = cis.name_prefix_offset;
+  if (!has_id_icon && cis.is_id && !requires_exact_data_name) {
+    cis.iconid = UI_icon_from_library(static_cast<const ID *>(cis.data));
+    char name_buf[UI_MAX_DRAW_STR];
     BKE_id_full_name_ui_prefix_get(
-        name_buf, static_cast<const ID *>(cis->data), false, UI_SEP_CHAR, &name_prefix_offset);
-    const int name_buf_len = strlen(name_buf);
-    BLI_assert(name_buf_len <= strlen(cis->name));
-    memcpy(cis->name, name_buf, name_buf_len + 1);
+        name_buf, static_cast<const ID *>(cis.data), false, UI_SEP_CHAR, &name_prefix_offset);
+    cis.name = name_buf;
   }
 
   return UI_search_item_add(items,
-                            cis->name,
-                            cis->data,
-                            cis->iconid,
-                            cis->has_sep_char ? int(UI_BUT_HAS_SEP_CHAR) : 0,
+                            cis.name,
+                            cis.data,
+                            cis.iconid,
+                            cis.has_sep_char ? int(UI_BUT_HAS_SEP_CHAR) : 0,
                             name_prefix_offset);
 }
 
 void ui_rna_collection_search_update_fn(
     const bContext *C, void *arg, const char *str, uiSearchItems *items, const bool is_first)
 {
+  using namespace blender;
   uiRNACollectionSearch *data = static_cast<uiRNACollectionSearch *>(arg);
   const int flag = RNA_property_flag(data->target_prop);
-  ListBase *items_list = MEM_cnew<ListBase>("items_list");
   const bool is_ptr_target = (RNA_property_type(data->target_prop) == PROP_POINTER);
   /* For non-pointer properties, UI code acts entirely based on the item's name. So the name has to
    * match the RNA name exactly. So only for pointer properties, the name can be modified to add
@@ -516,16 +489,14 @@ void ui_rna_collection_search_update_fn(
   const bool requires_exact_data_name = !is_ptr_target;
   const bool skip_filter = is_first;
   char name_buf[UI_MAX_DRAW_STR];
-  char *name;
   bool has_id_icon = false;
 
-  blender::ui::string_search::StringSearch<CollItemSearch> search;
+  /* The string search API requires pointer stability. */
+  Vector<std::unique_ptr<CollItemSearch>> items_list;
 
   if (data->search_prop != nullptr) {
     /* build a temporary list of relevant items first */
-    int item_index = 0;
     RNA_PROP_BEGIN (&data->search_ptr, itemptr, data->search_prop) {
-
       if (flag & PROP_ID_SELF_CHECK) {
         if (itemptr.data == data->target_ptr.owner_id) {
           continue;
@@ -544,6 +515,7 @@ void ui_rna_collection_search_update_fn(
       bool has_sep_char = false;
       const bool is_id = itemptr.type && RNA_struct_is_ID(itemptr.type);
 
+      char *name;
       if (is_id) {
         iconid = ui_id_icon_get(C, static_cast<ID *>(itemptr.data), false);
         if (!ELEM(iconid, 0, ICON_BLANK1)) {
@@ -562,29 +534,28 @@ void ui_rna_collection_search_update_fn(
           has_sep_char = ID_IS_LINKED(id);
         }
       }
+      else if (itemptr.type == &RNA_ActionSlot) {
+        PropertyRNA *prop = RNA_struct_find_property(&itemptr, "name_display");
+        name = RNA_property_string_get_alloc(&itemptr, prop, name_buf, sizeof(name_buf), nullptr);
+      }
       else {
         name = RNA_struct_name_get_alloc(&itemptr, name_buf, sizeof(name_buf), nullptr);
       }
 
       if (name) {
-        CollItemSearch *cis = MEM_cnew<CollItemSearch>(__func__);
+        auto cis = std::make_unique<CollItemSearch>();
         cis->data = itemptr.data;
-        cis->name = BLI_strdup(name);
-        cis->index = item_index;
+        cis->name = name;
+        cis->index = items_list.size();
         cis->iconid = iconid;
         cis->is_id = is_id;
         cis->name_prefix_offset = name_prefix_offset;
         cis->has_sep_char = has_sep_char;
-        if (!skip_filter) {
-          search.add(name, cis);
-        }
-        BLI_addtail(items_list, cis);
+        items_list.append(std::move(cis));
         if (name != name_buf) {
           MEM_freeN(name);
         }
       }
-
-      item_index++;
     }
     RNA_PROP_END;
   }
@@ -594,85 +565,66 @@ void ui_rna_collection_search_update_fn(
         data->target_prop);
     BLI_assert(search_flag & PROP_STRING_SEARCH_SUPPORTED);
 
-    struct SearchVisitUserData {
-      blender::string_search::StringSearch<CollItemSearch> *search;
-      bool skip_filter;
-      int item_index;
-      ListBase *items_list;
-      const char *func_id;
-    } user_data = {nullptr};
+    const bool show_extra_info = (G.debug_value == 102);
 
-    user_data.search = &search;
-    user_data.skip_filter = skip_filter;
-    user_data.items_list = items_list;
-    user_data.func_id = __func__;
+    RNA_property_string_search(C,
+                               &data->target_ptr,
+                               data->target_prop,
+                               str,
+                               [&](StringPropertySearchVisitParams visit_params) {
+                                 auto cis = std::make_unique<CollItemSearch>();
 
-    RNA_property_string_search(
-        C,
-        &data->target_ptr,
-        data->target_prop,
-        str,
-        [](void *user_data, const StringPropertySearchVisitParams *visit_params) {
-          const bool show_extra_info = (G.debug_value == 102);
+                                 cis->data = nullptr;
+                                 if (visit_params.info && show_extra_info) {
+                                   cis->name = fmt::format("{}" UI_SEP_CHAR_S "{}",
+                                                           visit_params.text,
+                                                           *visit_params.info);
+                                 }
+                                 else {
+                                   cis->name = std::move(visit_params.text);
+                                 }
 
-          SearchVisitUserData *search_data = (SearchVisitUserData *)user_data;
-          CollItemSearch *cis = MEM_cnew<CollItemSearch>(search_data->func_id);
-          cis->data = nullptr;
-          if (visit_params->info && show_extra_info) {
-            cis->name = BLI_sprintfN(
-                "%s" UI_SEP_CHAR_S "%s", visit_params->text, visit_params->info);
-          }
-          else {
-            cis->name = BLI_strdup(visit_params->text);
-          }
-          cis->index = search_data->item_index;
-          cis->iconid = ICON_NONE;
-          cis->is_id = false;
-          cis->name_prefix_offset = 0;
-          cis->has_sep_char = visit_params->info != nullptr;
-          if (!search_data->skip_filter) {
-            search_data->search->add(visit_params->text, cis);
-          }
-          BLI_addtail(search_data->items_list, cis);
-          search_data->item_index++;
-        },
-        (void *)&user_data);
+                                 cis->index = items_list.size();
+                                 cis->iconid = visit_params.icon_id.value_or(ICON_NONE);
+                                 cis->is_id = false;
+                                 cis->name_prefix_offset = 0;
+                                 cis->has_sep_char = visit_params.info.has_value();
+                                 items_list.append(std::move(cis));
+                               });
 
     if (search_flag & PROP_STRING_SEARCH_SORT) {
-      BLI_listbase_sort(items_list, [](const void *a_, const void *b_) -> int {
-        const CollItemSearch *cis_a = (const CollItemSearch *)a_;
-        const CollItemSearch *cis_b = (const CollItemSearch *)b_;
-        return BLI_strcasecmp_natural(cis_a->name, cis_b->name);
-      });
-      int i = 0;
-      LISTBASE_FOREACH (CollItemSearch *, cis, items_list) {
-        cis->index = i;
-        i++;
+      std::sort(
+          items_list.begin(),
+          items_list.end(),
+          [](const std::unique_ptr<CollItemSearch> &a, const std::unique_ptr<CollItemSearch> &b) {
+            return BLI_strcasecmp_natural(a->name.c_str(), b->name.c_str()) < 0;
+          });
+      for (const int i : items_list.index_range()) {
+        items_list[i]->index = i;
       }
     }
   }
 
   if (skip_filter) {
-    LISTBASE_FOREACH (CollItemSearch *, cis, items_list) {
-      if (!add_collection_search_item(cis, requires_exact_data_name, has_id_icon, items)) {
+    for (std::unique_ptr<CollItemSearch> &cis : items_list) {
+      if (!add_collection_search_item(*cis, requires_exact_data_name, has_id_icon, items)) {
         break;
       }
     }
   }
   else {
-    const blender::Vector<CollItemSearch *> filtered_items = search.query(str);
+    ui::string_search::StringSearch<CollItemSearch> search;
+    for (std::unique_ptr<CollItemSearch> &cis : items_list) {
+      search.add(cis->name, cis.get());
+    }
+
+    const Vector<CollItemSearch *> filtered_items = search.query(str);
     for (CollItemSearch *cis : filtered_items) {
-      if (!add_collection_search_item(cis, requires_exact_data_name, has_id_icon, items)) {
+      if (!add_collection_search_item(*cis, requires_exact_data_name, has_id_icon, items)) {
         break;
       }
     }
   }
-
-  LISTBASE_FOREACH (CollItemSearch *, cis, items_list) {
-    MEM_freeN(cis->name);
-  }
-  BLI_freelistN(items_list);
-  MEM_freeN(items_list);
 }
 
 int UI_icon_from_id(const ID *id)
@@ -724,13 +676,13 @@ int UI_icon_from_report_type(int type)
 int UI_icon_colorid_from_report_type(int type)
 {
   if (type & RPT_ERROR_ALL) {
-    return TH_INFO_ERROR;
+    return TH_ERROR;
   }
   if (type & RPT_WARNING_ALL) {
-    return TH_INFO_WARNING;
+    return TH_WARNING;
   }
   if (type & RPT_INFO_ALL) {
-    return TH_INFO_INFO;
+    return TH_INFO;
   }
   if (type & RPT_DEBUG_ALL) {
     return TH_INFO_DEBUG;
@@ -741,7 +693,7 @@ int UI_icon_colorid_from_report_type(int type)
   if (type & RPT_OPERATOR) {
     return TH_INFO_OPERATOR;
   }
-  return TH_INFO_WARNING;
+  return TH_WARNING;
 }
 
 int UI_text_colorid_from_report_type(int type)
@@ -810,9 +762,7 @@ int UI_calc_float_precision(int prec, double value)
         dec_flag = dec_flag >> 1;
       }
 
-      if (test_prec > prec) {
-        prec = test_prec;
-      }
+      prec = std::max(test_prec, prec);
     }
   }
 
@@ -821,35 +771,27 @@ int UI_calc_float_precision(int prec, double value)
   return prec;
 }
 
-bool UI_but_online_manual_id(const uiBut *but, char *r_str, size_t str_maxncpy)
+std::optional<std::string> UI_but_online_manual_id(const uiBut *but)
 {
-  if (but->rnapoin.owner_id && but->rnapoin.data && but->rnaprop) {
-    BLI_snprintf(r_str,
-                 str_maxncpy,
-                 "%s.%s",
-                 RNA_struct_identifier(but->rnapoin.type),
-                 RNA_property_identifier(but->rnaprop));
-    return true;
+  if (but->rnapoin.data && but->rnaprop) {
+    return fmt::format(
+        "{}.{}", RNA_struct_identifier(but->rnapoin.type), RNA_property_identifier(but->rnaprop));
   }
   if (but->optype) {
-    WM_operator_py_idname(r_str, but->optype->idname);
-    return true;
+    char idname[OP_MAX_TYPENAME];
+    const size_t idname_len = WM_operator_py_idname(idname, but->optype->idname);
+    return std::string(idname, idname_len);
   }
 
-  *r_str = '\0';
-  return false;
+  return std::nullopt;
 }
 
-bool UI_but_online_manual_id_from_active(const bContext *C, char *r_str, size_t str_maxncpy)
+std::optional<std::string> UI_but_online_manual_id_from_active(const bContext *C)
 {
-  uiBut *but = UI_context_active_but_get(C);
-
-  if (but) {
-    return UI_but_online_manual_id(but, r_str, str_maxncpy);
+  if (uiBut *but = UI_context_active_but_get(C)) {
+    return UI_but_online_manual_id(but);
   }
-
-  *r_str = '\0';
-  return false;
+  return std::nullopt;
 }
 
 /* -------------------------------------------------------------------- */
@@ -959,7 +901,7 @@ struct uiButStoreElem {
 
 uiButStore *UI_butstore_create(uiBlock *block)
 {
-  uiButStore *bs_handle = MEM_cnew<uiButStore>(__func__);
+  uiButStore *bs_handle = MEM_callocN<uiButStore>(__func__);
 
   bs_handle->block = block;
   BLI_addtail(&block->butstore, bs_handle);
@@ -969,15 +911,14 @@ uiButStore *UI_butstore_create(uiBlock *block)
 
 void UI_butstore_free(uiBlock *block, uiButStore *bs_handle)
 {
-  /* Workaround for button store being moved into new block,
+  /* NOTE(@ideasman42): Workaround for button store being moved into new block,
    * which then can't use the previous buttons state
-   * ('ui_but_update_from_old_block' fails to find a match),
+   * (#ui_but_update_from_old_block fails to find a match),
    * keeping the active button in the old block holding a reference
    * to the button-state in the new block: see #49034.
    *
    * Ideally we would manage moving the 'uiButStore', keeping a correct state.
-   * All things considered this is the most straightforward fix - Campbell.
-   */
+   * All things considered this is the most straightforward fix. */
   if (block != bs_handle->block && bs_handle->block != nullptr) {
     block = bs_handle->block;
   }
@@ -989,9 +930,9 @@ void UI_butstore_free(uiBlock *block, uiButStore *bs_handle)
   MEM_freeN(bs_handle);
 }
 
-bool UI_butstore_is_valid(uiButStore *bs)
+bool UI_butstore_is_valid(uiButStore *bs_handle)
 {
-  return (bs->block != nullptr);
+  return (bs_handle->block != nullptr);
 }
 
 bool UI_butstore_is_registered(uiBlock *block, uiBut *but)
@@ -1009,7 +950,7 @@ bool UI_butstore_is_registered(uiBlock *block, uiBut *but)
 
 void UI_butstore_register(uiButStore *bs_handle, uiBut **but_p)
 {
-  uiButStoreElem *bs_elem = MEM_cnew<uiButStoreElem>(__func__);
+  uiButStoreElem *bs_elem = MEM_callocN<uiButStoreElem>(__func__);
   BLI_assert(*but_p);
   bs_elem->but_p = but_p;
 
@@ -1099,13 +1040,13 @@ void UI_butstore_update(uiBlock *block)
 /**
  * Follow the logic from #wm_keymap_item_find_in_keymap.
  */
-static bool ui_key_event_property_match(const char *opname,
+static bool ui_key_event_property_match(const StringRefNull opname,
                                         IDProperty *properties,
                                         const bool is_strict,
                                         wmOperatorType *ui_optype,
                                         PointerRNA *ui_opptr)
 {
-  if (!STREQ(ui_optype->idname, opname)) {
+  if (ui_optype->idname != opname) {
     return false;
   }
 
@@ -1123,40 +1064,39 @@ static bool ui_key_event_property_match(const char *opname,
   return match;
 }
 
-const char *UI_key_event_operator_string(const bContext *C,
-                                         const char *opname,
-                                         IDProperty *properties,
-                                         const bool is_strict,
-                                         char *result,
-                                         const int result_maxncpy)
+std::optional<std::string> UI_key_event_operator_string(const bContext *C,
+                                                        const StringRefNull opname,
+                                                        IDProperty *properties,
+                                                        const bool is_strict)
 {
   /* NOTE: currently only actions on UI Lists are supported (for the asset manager).
    * Other kinds of events can be supported as needed. */
 
   ARegion *region = CTX_wm_region(C);
   if (region == nullptr) {
-    return nullptr;
+    return std::nullopt;
   }
 
   /* Early exit regions which don't have UI-Lists. */
-  if ((region->type->keymapflag & ED_KEYMAP_UI) == 0) {
-    return nullptr;
+  if ((region->runtime->type->keymapflag & ED_KEYMAP_UI) == 0) {
+    return std::nullopt;
   }
 
   uiBut *but = UI_region_active_but_get(region);
   if (but == nullptr) {
-    return nullptr;
+    return std::nullopt;
   }
 
   if (but->type != UI_BTYPE_PREVIEW_TILE) {
-    return nullptr;
+    return std::nullopt;
   }
 
   short event_val = KM_NOTHING;
   short event_type = KM_NOTHING;
 
   uiBut *listbox = nullptr;
-  LISTBASE_FOREACH_BACKWARD (uiBut *, but_iter, &but->block->buttons) {
+  for (int i = but->block->buttons.size() - 1; i >= 0; i--) {
+    uiBut *but_iter = but->block->buttons[i].get();
     if ((but_iter->type == UI_BTYPE_LISTBOX) && ui_but_contains_rect(but_iter, &but->rect)) {
       listbox = but_iter;
       break;
@@ -1189,12 +1129,18 @@ const char *UI_key_event_operator_string(const bContext *C,
   }
 
   if ((event_val != KM_NOTHING) && (event_type != KM_NOTHING)) {
-    WM_keymap_item_raw_to_string(
-        false, false, false, false, 0, event_val, event_type, false, result, result_maxncpy);
-    return result;
+    return WM_keymap_item_raw_to_string(KM_NOTHING,
+                                        KM_NOTHING,
+                                        KM_NOTHING,
+                                        KM_NOTHING,
+                                        KM_NOTHING,
+                                        0,
+                                        event_val,
+                                        event_type,
+                                        false);
   }
 
-  return nullptr;
+  return std::nullopt;
 }
 
 /** \} */
