@@ -8,23 +8,23 @@
  */
 
 #include "DNA_brush_types.h"
-#include "DNA_color_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_space_types.h"
 
 #include "BLI_math_color.h"
+#include "BLI_math_vector.h"
 
 #include "BKE_brush.hh"
-#include "BKE_context.h"
-#include "BKE_layer.h"
+#include "BKE_context.hh"
+#include "BKE_layer.hh"
 #include "BKE_paint.hh"
-#include "BKE_undo_system.h"
+#include "BKE_undo_system.hh"
 
 #include "ED_paint.hh"
 #include "ED_view3d.hh"
 
-#include "GPU_immediate.h"
-#include "GPU_state.h"
+#include "GPU_immediate.hh"
+#include "GPU_state.hh"
 
 #include "MEM_guardedalloc.h"
 
@@ -61,6 +61,7 @@ class AbstractPaintMode {
   virtual void paint_stroke_done(void *stroke_handle) = 0;
   virtual void paint_gradient_fill(const bContext *C,
                                    const Scene *scene,
+                                   const Paint *paint,
                                    Brush *brush,
                                    PaintStroke *stroke,
                                    void *stroke_handle,
@@ -68,6 +69,7 @@ class AbstractPaintMode {
                                    float mouse_end[2]) = 0;
   virtual void paint_bucket_fill(const bContext *C,
                                  const Scene *scene,
+                                 const Paint *paint,
                                  Brush *brush,
                                  PaintStroke *stroke,
                                  void *stroke_handle,
@@ -107,6 +109,7 @@ class ImagePaintMode : public AbstractPaintMode {
 
   void paint_gradient_fill(const bContext *C,
                            const Scene * /*scene*/,
+                           const Paint * /*paint*/,
                            Brush *brush,
                            PaintStroke * /*stroke*/,
                            void *stroke_handle,
@@ -118,6 +121,7 @@ class ImagePaintMode : public AbstractPaintMode {
 
   void paint_bucket_fill(const bContext *C,
                          const Scene *scene,
+                         const Paint *paint,
                          Brush *brush,
                          PaintStroke *stroke,
                          void *stroke_handle,
@@ -126,10 +130,10 @@ class ImagePaintMode : public AbstractPaintMode {
   {
     float color[3];
     if (paint_stroke_inverted(stroke)) {
-      srgb_to_linearrgb_v3_v3(color, BKE_brush_secondary_color_get(scene, brush));
+      copy_v3_v3(color, BKE_brush_secondary_color_get(scene, paint, brush));
     }
     else {
-      srgb_to_linearrgb_v3_v3(color, BKE_brush_color_get(scene, brush));
+      copy_v3_v3(color, BKE_brush_color_get(scene, paint, brush));
     }
     paint_2d_bucket_fill(C, color, brush, mouse_start, mouse_end, stroke_handle);
   }
@@ -167,29 +171,32 @@ class ProjectionPaintMode : public AbstractPaintMode {
 
   void paint_gradient_fill(const bContext *C,
                            const Scene *scene,
+                           const Paint *paint,
                            Brush *brush,
                            PaintStroke *stroke,
                            void *stroke_handle,
                            float mouse_start[2],
                            float mouse_end[2]) override
   {
-    paint_fill(C, scene, brush, stroke, stroke_handle, mouse_start, mouse_end);
+    paint_fill(C, scene, paint, brush, stroke, stroke_handle, mouse_start, mouse_end);
   }
 
   void paint_bucket_fill(const bContext *C,
                          const Scene *scene,
+                         const Paint *paint,
                          Brush *brush,
                          PaintStroke *stroke,
                          void *stroke_handle,
                          float mouse_start[2],
                          float mouse_end[2]) override
   {
-    paint_fill(C, scene, brush, stroke, stroke_handle, mouse_start, mouse_end);
+    paint_fill(C, scene, paint, brush, stroke, stroke_handle, mouse_start, mouse_end);
   }
 
  private:
   void paint_fill(const bContext *C,
                   const Scene *scene,
+                  const Paint * /*paint*/,
                   Brush *brush,
                   PaintStroke *stroke,
                   void *stroke_handle,
@@ -210,7 +217,7 @@ class ProjectionPaintMode : public AbstractPaintMode {
   }
 };
 
-struct PaintOperation {
+struct PaintOperation : public PaintModeData {
   AbstractPaintMode *mode = nullptr;
 
   void *stroke_handle = nullptr;
@@ -223,7 +230,7 @@ struct PaintOperation {
   ViewContext vc = {nullptr};
 
   PaintOperation() = default;
-  ~PaintOperation()
+  ~PaintOperation() override
   {
     MEM_delete(mode);
     mode = nullptr;
@@ -235,7 +242,10 @@ struct PaintOperation {
   }
 };
 
-static void gradient_draw_line(bContext * /*C*/, int x, int y, void *customdata)
+static void gradient_draw_line(bContext * /*C*/,
+                               const blender::int2 &xy,
+                               const blender::float2 & /*tilt*/,
+                               void *customdata)
 {
   PaintOperation *pop = (PaintOperation *)customdata;
 
@@ -244,7 +254,7 @@ static void gradient_draw_line(bContext * /*C*/, int x, int y, void *customdata)
     GPU_blend(GPU_BLEND_ALPHA);
 
     GPUVertFormat *format = immVertexFormat();
-    uint pos = GPU_vertformat_attr_add(format, "pos", GPU_COMP_I32, 2, GPU_FETCH_INT_TO_FLOAT);
+    uint pos = GPU_vertformat_attr_add(format, "pos", GPU_COMP_F32, 2, GPU_FETCH_FLOAT);
 
     ARegion *region = pop->vc.region;
 
@@ -254,8 +264,8 @@ static void gradient_draw_line(bContext * /*C*/, int x, int y, void *customdata)
     immUniformColor4ub(0, 0, 0, 255);
 
     immBegin(GPU_PRIM_LINES, 2);
-    immVertex2i(pos, x, y);
-    immVertex2i(
+    immVertex2fv(pos, blender::float2(xy));
+    immVertex2f(
         pos, pop->startmouse[0] + region->winrct.xmin, pop->startmouse[1] + region->winrct.ymin);
     immEnd();
 
@@ -263,8 +273,8 @@ static void gradient_draw_line(bContext * /*C*/, int x, int y, void *customdata)
     immUniformColor4ub(255, 255, 255, 255);
 
     immBegin(GPU_PRIM_LINES, 2);
-    immVertex2i(pos, x, y);
-    immVertex2i(
+    immVertex2fv(pos, blender::float2(xy));
+    immVertex2f(
         pos, pop->startmouse[0] + region->winrct.xmin, pop->startmouse[1] + region->winrct.ymin);
     immEnd();
 
@@ -275,15 +285,17 @@ static void gradient_draw_line(bContext * /*C*/, int x, int y, void *customdata)
   }
 }
 
-static PaintOperation *texture_paint_init(bContext *C, wmOperator *op, const float mouse[2])
+static std::unique_ptr<PaintOperation> texture_paint_init(bContext *C,
+                                                          wmOperator *op,
+                                                          const float mouse[2])
 {
   Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
   Scene *scene = CTX_data_scene(C);
   ToolSettings *settings = scene->toolsettings;
-  PaintOperation *pop = MEM_new<PaintOperation>("PaintOperation"); /* caller frees */
+  std::unique_ptr<PaintOperation> pop = std::make_unique<PaintOperation>();
   Brush *brush = BKE_paint_brush(&settings->imapaint.paint);
   int mode = RNA_enum_get(op->ptr, "mode");
-  ED_view3d_viewcontext_init(C, &pop->vc, depsgraph);
+  pop->vc = ED_view3d_viewcontext_init(C, depsgraph);
 
   copy_v2_v2(pop->prevmouse, mouse);
   copy_v2_v2(pop->startmouse, mouse);
@@ -295,9 +307,8 @@ static PaintOperation *texture_paint_init(bContext *C, wmOperator *op, const flo
   /* initialize from context */
   if (CTX_wm_region_view3d(C)) {
     bool uvs, mat, tex, stencil;
-    if (!ED_paint_proj_mesh_data_check(scene, ob, &uvs, &mat, &tex, &stencil)) {
+    if (!ED_paint_proj_mesh_data_check(*scene, *ob, &uvs, &mat, &tex, &stencil)) {
       ED_paint_data_warning(op->reports, uvs, mat, tex, stencil);
-      MEM_delete(pop);
       WM_event_add_notifier(C, NC_SCENE | ND_TOOLSETTINGS, nullptr);
       return nullptr;
     }
@@ -309,23 +320,24 @@ static PaintOperation *texture_paint_init(bContext *C, wmOperator *op, const flo
 
   pop->stroke_handle = pop->mode->paint_new_stroke(C, op, ob, mouse, mode);
   if (!pop->stroke_handle) {
-    MEM_delete(pop);
     return nullptr;
   }
 
-  if ((brush->imagepaint_tool == PAINT_TOOL_FILL) && (brush->flag & BRUSH_USE_GRADIENT)) {
+  if ((brush->image_brush_type == IMAGE_PAINT_BRUSH_TYPE_FILL) &&
+      (brush->flag & BRUSH_USE_GRADIENT))
+  {
     pop->cursor = WM_paint_cursor_activate(
-        SPACE_TYPE_ANY, RGN_TYPE_ANY, ED_image_tools_paint_poll, gradient_draw_line, pop);
+        SPACE_TYPE_ANY, RGN_TYPE_ANY, ED_image_tools_paint_poll, gradient_draw_line, pop.get());
   }
 
   settings->imapaint.flag |= IMAGEPAINT_DRAWING;
-  ED_image_undo_push_begin(op->type->name, PAINT_MODE_TEXTURE_2D);
+  ED_image_undo_push_begin(op->type->name, PaintMode::Texture2D);
 
   return pop;
 }
 
 static void paint_stroke_update_step(bContext *C,
-                                     wmOperator * /*op*/,
+                                     wmOperator *op,
                                      PaintStroke *stroke,
                                      PointerRNA *itemptr)
 {
@@ -348,11 +360,11 @@ static void paint_stroke_update_step(bContext *C,
 
   RNA_float_get_array(itemptr, "mouse", mouse);
   pressure = RNA_float_get(itemptr, "pressure");
-  eraser = RNA_boolean_get(itemptr, "pen_flip");
+  eraser = RNA_boolean_get(op->ptr, "pen_flip");
   size = RNA_float_get(itemptr, "size");
 
   /* stroking with fill tool only acts on stroke end */
-  if (brush->imagepaint_tool == PAINT_TOOL_FILL) {
+  if (brush->image_brush_type == IMAGE_PAINT_BRUSH_TYPE_FILL) {
     copy_v2_v2(pop->prevmouse, mouse);
     return;
   }
@@ -389,18 +401,19 @@ static void paint_stroke_done(const bContext *C, PaintStroke *stroke)
   Scene *scene = CTX_data_scene(C);
   ToolSettings *toolsettings = scene->toolsettings;
   PaintOperation *pop = static_cast<PaintOperation *>(paint_stroke_mode_data(stroke));
+  const Paint *paint = BKE_paint_get_active_from_context(C);
   Brush *brush = BKE_paint_brush(&toolsettings->imapaint.paint);
 
   toolsettings->imapaint.flag &= ~IMAGEPAINT_DRAWING;
 
-  if (brush->imagepaint_tool == PAINT_TOOL_FILL) {
+  if (brush->image_brush_type == IMAGE_PAINT_BRUSH_TYPE_FILL) {
     if (brush->flag & BRUSH_USE_GRADIENT) {
       pop->mode->paint_gradient_fill(
-          C, scene, brush, stroke, pop->stroke_handle, pop->startmouse, pop->prevmouse);
+          C, scene, paint, brush, stroke, pop->stroke_handle, pop->startmouse, pop->prevmouse);
     }
     else {
       pop->mode->paint_bucket_fill(
-          C, scene, brush, stroke, pop->stroke_handle, pop->startmouse, pop->prevmouse);
+          C, scene, paint, brush, stroke, pop->stroke_handle, pop->startmouse, pop->prevmouse);
     }
   }
   pop->mode->paint_stroke_done(pop->stroke_handle);
@@ -423,12 +436,11 @@ static void paint_stroke_done(const bContext *C, PaintStroke *stroke)
                 pop->s.warnpackedfile);
   }
 #endif
-  MEM_delete(pop);
 }
 
 static bool paint_stroke_test_start(bContext *C, wmOperator *op, const float mouse[2])
 {
-  PaintOperation *pop;
+  std::unique_ptr<PaintOperation> pop;
 
   /* TODO: Should avoid putting this here. Instead, last position should be requested
    * from stroke system. */
@@ -437,15 +449,13 @@ static bool paint_stroke_test_start(bContext *C, wmOperator *op, const float mou
     return false;
   }
 
-  paint_stroke_set_mode_data(static_cast<PaintStroke *>(op->customdata), pop);
+  paint_stroke_set_mode_data(static_cast<PaintStroke *>(op->customdata), std::move(pop));
 
   return true;
 }
 
-static int paint_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus paint_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
-  int retval;
-
   op->customdata = paint_stroke_new(C,
                                     op,
                                     nullptr,
@@ -455,20 +465,22 @@ static int paint_invoke(bContext *C, wmOperator *op, const wmEvent *event)
                                     paint_stroke_done,
                                     event->type);
 
-  if ((retval = op->type->modal(C, op, event)) == OPERATOR_FINISHED) {
+  const wmOperatorStatus retval = op->type->modal(C, op, event);
+  OPERATOR_RETVAL_CHECK(retval);
+
+  if (retval == OPERATOR_FINISHED) {
     paint_stroke_free(C, op, static_cast<PaintStroke *>(op->customdata));
     return OPERATOR_FINISHED;
   }
   /* add modal handler */
   WM_event_add_modal_handler(C, op);
 
-  OPERATOR_RETVAL_CHECK(retval);
   BLI_assert(retval == OPERATOR_RUNNING_MODAL);
 
   return OPERATOR_RUNNING_MODAL;
 }
 
-static int paint_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus paint_exec(bContext *C, wmOperator *op)
 {
   PropertyRNA *strokeprop;
   PointerRNA firstpoint;
@@ -482,19 +494,36 @@ static int paint_exec(bContext *C, wmOperator *op)
 
   RNA_float_get_array(&firstpoint, "mouse", mouse);
 
-  op->customdata = paint_stroke_new(C,
-                                    op,
-                                    nullptr,
-                                    paint_stroke_test_start,
-                                    paint_stroke_update_step,
-                                    paint_stroke_redraw,
-                                    paint_stroke_done,
-                                    0);
+  PaintStroke *stroke = paint_stroke_new(C,
+                                         op,
+                                         nullptr,
+                                         paint_stroke_test_start,
+                                         paint_stroke_update_step,
+                                         paint_stroke_redraw,
+                                         paint_stroke_done,
+                                         0);
+  op->customdata = stroke;
+
+  /* Make sure we have proper coordinates for sampling (mask) textures -- these get stored in
+   * #UnifiedPaintSettings -- as well as support randomness and jitter. */
+  Scene &scene = *CTX_data_scene(C);
+  PaintMode mode = BKE_paintmode_get_active_from_context(C);
+  Paint &paint = *BKE_paint_get_active_from_context(C);
+  const Brush &brush = *BKE_paint_brush_for_read(&paint);
+  float pressure;
+  pressure = RNA_float_get(&firstpoint, "pressure");
+  float mouse_out[2];
+  bool dummy;
+  float dummy_location[3];
+
+  paint_stroke_jitter_pos(scene, *stroke, mode, brush, pressure, mouse, mouse_out);
+  paint_brush_update(C, brush, mode, stroke, mouse, mouse_out, pressure, dummy_location, &dummy);
+
   /* frees op->customdata */
   return paint_stroke_exec(C, op, static_cast<PaintStroke *>(op->customdata));
 }
 
-static int paint_modal(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus paint_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
   return paint_stroke_modal(C, op, event, reinterpret_cast<PaintStroke **>(&op->customdata));
 }
@@ -514,7 +543,7 @@ void PAINT_OT_image_paint(wmOperatorType *ot)
   ot->idname = "PAINT_OT_image_paint";
   ot->description = "Paint a stroke into the image";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = paint_invoke;
   ot->modal = paint_modal;
   ot->exec = paint_exec;
