@@ -14,6 +14,7 @@
 
 #include "MEM_guardedalloc.h"
 
+#include "BLI_listbase.h"
 #include "BLI_math_geom.h"
 #include "BLI_math_matrix.h"
 #include "BLI_math_rotation.h"
@@ -78,7 +79,7 @@ static bool view3d_lock_poll(bContext *C)
 /** \name View Lock Clear Operator
  * \{ */
 
-static int view_lock_clear_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus view_lock_clear_exec(bContext *C, wmOperator * /*op*/)
 {
   View3D *v3d = CTX_wm_view3d(C);
 
@@ -101,7 +102,7 @@ void VIEW3D_OT_view_lock_clear(wmOperatorType *ot)
   ot->description = "Clear all view locking";
   ot->idname = "VIEW3D_OT_view_lock_clear";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = view_lock_clear_exec;
   ot->poll = ED_operator_region_view3d_active;
 
@@ -115,7 +116,7 @@ void VIEW3D_OT_view_lock_clear(wmOperatorType *ot)
 /** \name View Lock to Active Operator
  * \{ */
 
-static int view_lock_to_active_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus view_lock_to_active_exec(bContext *C, wmOperator * /*op*/)
 {
   View3D *v3d = CTX_wm_view3d(C);
   Object *obact = CTX_data_active_object(C);
@@ -128,7 +129,7 @@ static int view_lock_to_active_exec(bContext *C, wmOperator * /*op*/)
     if (obact && obact->type == OB_ARMATURE) {
       if (obact->mode & OB_MODE_POSE) {
         Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
-        Object *obact_eval = DEG_get_evaluated_object(depsgraph, obact);
+        Object *obact_eval = DEG_get_evaluated(depsgraph, obact);
         bPoseChannel *pcham_act = BKE_pose_channel_active_if_bonecoll_visible(obact_eval);
         if (pcham_act) {
           STRNCPY(v3d->ob_center_bone, pcham_act->name);
@@ -158,7 +159,7 @@ void VIEW3D_OT_view_lock_to_active(wmOperatorType *ot)
   ot->description = "Lock the view to the active object/bone";
   ot->idname = "VIEW3D_OT_view_lock_to_active";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = view_lock_to_active_exec;
   ot->poll = ED_operator_region_view3d_active;
 
@@ -172,7 +173,7 @@ void VIEW3D_OT_view_lock_to_active(wmOperatorType *ot)
 /** \name Frame Camera Bounds Operator
  * \{ */
 
-static int view3d_center_camera_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus view3d_center_camera_exec(bContext *C, wmOperator * /*op*/)
 {
   Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
   Scene *scene = CTX_data_scene(C);
@@ -210,7 +211,7 @@ void VIEW3D_OT_view_center_camera(wmOperatorType *ot)
   ot->description = "Center the camera view, resizing the view to fit its bounds";
   ot->idname = "VIEW3D_OT_view_center_camera";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = view3d_center_camera_exec;
   ot->poll = view3d_camera_user_poll;
 
@@ -224,7 +225,7 @@ void VIEW3D_OT_view_center_camera(wmOperatorType *ot)
 /** \name View Lock Center Operator
  * \{ */
 
-static int view3d_center_lock_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus view3d_center_lock_exec(bContext *C, wmOperator * /*op*/)
 {
   RegionView3D *rv3d = CTX_wm_region_view3d(C);
 
@@ -242,7 +243,7 @@ void VIEW3D_OT_view_center_lock(wmOperatorType *ot)
   ot->description = "Center the view lock offset";
   ot->idname = "VIEW3D_OT_view_center_lock";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = view3d_center_lock_exec;
   ot->poll = view3d_lock_poll;
 
@@ -256,7 +257,7 @@ void VIEW3D_OT_view_center_lock(wmOperatorType *ot)
 /** \name Set Render Border Operator
  * \{ */
 
-static int render_border_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus render_border_exec(bContext *C, wmOperator *op)
 {
   View3D *v3d = CTX_wm_view3d(C);
   ARegion *region = CTX_wm_region(C);
@@ -337,7 +338,7 @@ void VIEW3D_OT_render_border(wmOperatorType *ot)
   ot->description = "Set the boundaries of the border render and enable border render";
   ot->idname = "VIEW3D_OT_render_border";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = WM_gesture_box_invoke;
   ot->exec = render_border_exec;
   ot->modal = WM_gesture_box_modal;
@@ -360,7 +361,7 @@ void VIEW3D_OT_render_border(wmOperatorType *ot)
 /** \name Clear Render Border Operator
  * \{ */
 
-static int clear_render_border_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus clear_render_border_exec(bContext *C, wmOperator *op)
 {
   View3D *v3d = CTX_wm_view3d(C);
   RegionView3D *rv3d = ED_view3d_context_rv3d(C);
@@ -400,7 +401,7 @@ void VIEW3D_OT_clear_render_border(wmOperatorType *ot)
   ot->description = "Clear the boundaries of the border render and disable border render";
   ot->idname = "VIEW3D_OT_clear_render_border";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = clear_render_border_exec;
   ot->poll = ED_operator_view3d_active;
 
@@ -435,7 +436,7 @@ static void view3d_set_1_to_1_viewborder(Scene *scene,
   CLAMP(rv3d->camzoom, RV3D_CAMZOOM_MIN, RV3D_CAMZOOM_MAX);
 }
 
-static int view3d_zoom_1_to_1_camera_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus view3d_zoom_1_to_1_camera_exec(bContext *C, wmOperator * /*op*/)
 {
   Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
   Scene *scene = CTX_data_scene(C);
@@ -463,7 +464,7 @@ void VIEW3D_OT_zoom_camera_1_to_1(wmOperatorType *ot)
   ot->description = "Match the camera to 1:1 to the render output";
   ot->idname = "VIEW3D_OT_zoom_camera_1_to_1";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = view3d_zoom_1_to_1_camera_exec;
   ot->poll = view3d_camera_user_poll;
 
@@ -477,7 +478,7 @@ void VIEW3D_OT_zoom_camera_1_to_1(wmOperatorType *ot)
 /** \name View Toggle Perspective/Orthographic Operator
  * \{ */
 
-static int viewpersportho_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus viewpersportho_exec(bContext *C, wmOperator * /*op*/)
 {
   View3D *v3d;
   ARegion *region;
@@ -510,7 +511,7 @@ void VIEW3D_OT_view_persportho(wmOperatorType *ot)
   ot->description = "Switch the current view from perspective/orthographic projection";
   ot->idname = "VIEW3D_OT_view_persportho";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = viewpersportho_exec;
   ot->poll = ED_operator_rv3d_user_region_poll;
 
@@ -526,7 +527,9 @@ void VIEW3D_OT_view_persportho(wmOperatorType *ot)
  * Wraps walk/fly modes.
  * \{ */
 
-static int view3d_navigate_invoke(bContext *C, wmOperator * /*op*/, const wmEvent *event)
+static wmOperatorStatus view3d_navigate_invoke(bContext *C,
+                                               wmOperator * /*op*/,
+                                               const wmEvent *event)
 {
   eViewNavigation_Method mode = eViewNavigation_Method(U.navigation_mode);
 
@@ -551,7 +554,7 @@ void VIEW3D_OT_navigate(wmOperatorType *ot)
       "Interactively navigate around the scene (uses the mode (walk/fly) preference)";
   ot->idname = "VIEW3D_OT_navigate";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = view3d_navigate_invoke;
   ot->poll = ED_operator_view3d_active;
 }
@@ -576,7 +579,7 @@ static Camera *background_image_camera_from_context(bContext *C)
   return static_cast<Camera *>(CTX_data_pointer_get_type(C, "camera", &RNA_Camera).data);
 }
 
-static int camera_background_image_add_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus camera_background_image_add_exec(bContext *C, wmOperator *op)
 {
   Camera *cam = background_image_camera_from_context(C);
   Image *ima;
@@ -608,7 +611,7 @@ void VIEW3D_OT_camera_background_image_add(wmOperatorType *ot)
   ot->description = "Add a new background image to the active camera";
   ot->idname = "VIEW3D_OT_camera_background_image_add";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = camera_background_image_add_exec;
   ot->poll = camera_background_image_add_poll;
 
@@ -618,13 +621,14 @@ void VIEW3D_OT_camera_background_image_add(wmOperatorType *ot)
   /* properties */
   PropertyRNA *prop = RNA_def_string(
       ot->srna, "filepath", nullptr, FILE_MAX, "Filepath", "Path to image file");
-  RNA_def_property_flag(prop, PropertyFlag(PROP_HIDDEN | PROP_SKIP_SAVE));
+  RNA_def_property_subtype(prop, PROP_FILEPATH);
+  RNA_def_property_flag(prop, PROP_HIDDEN | PROP_SKIP_SAVE | PROP_PATH_SUPPORTS_BLEND_RELATIVE);
   prop = RNA_def_boolean(ot->srna,
                          "relative_path",
                          true,
                          "Relative Path",
                          "Select the file relative to the blend file");
-  RNA_def_property_flag(prop, PropertyFlag(PROP_HIDDEN | PROP_SKIP_SAVE));
+  RNA_def_property_flag(prop, PROP_HIDDEN | PROP_SKIP_SAVE);
 
   WM_operator_properties_id_lookup(ot, true);
 }
@@ -635,7 +639,7 @@ void VIEW3D_OT_camera_background_image_add(wmOperatorType *ot)
 /** \name Background Image Remove Operator
  * \{ */
 
-static int camera_background_image_remove_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus camera_background_image_remove_exec(bContext *C, wmOperator *op)
 {
   Camera *cam = static_cast<Camera *>(CTX_data_pointer_get_type(C, "camera", &RNA_Camera).data);
   const int index = RNA_int_get(op->ptr, "index");
@@ -674,7 +678,7 @@ void VIEW3D_OT_camera_background_image_remove(wmOperatorType *ot)
   ot->description = "Remove a background image from the camera";
   ot->idname = "VIEW3D_OT_camera_background_image_remove";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = camera_background_image_remove_exec;
   ot->poll = ED_operator_camera_poll;
 
@@ -692,7 +696,7 @@ void VIEW3D_OT_camera_background_image_remove(wmOperatorType *ot)
 /** \name Drop World Operator
  * \{ */
 
-static int drop_world_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus drop_world_exec(bContext *C, wmOperator *op)
 {
   Main *bmain = CTX_data_main(C);
   Scene *scene = CTX_data_scene(C);
@@ -727,7 +731,7 @@ void VIEW3D_OT_drop_world(wmOperatorType *ot)
   ot->description = "Drop a world into the scene";
   ot->idname = "VIEW3D_OT_drop_world";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = drop_world_exec;
   ot->poll = drop_world_poll;
 
@@ -769,7 +773,7 @@ void ED_view3d_clipping_local(RegionView3D *rv3d, const float mat[4][4])
   }
 }
 
-static int view3d_clipping_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus view3d_clipping_exec(bContext *C, wmOperator *op)
 {
   ARegion *region = CTX_wm_region(C);
   RegionView3D *rv3d = CTX_wm_region_view3d(C);
@@ -778,7 +782,7 @@ static int view3d_clipping_exec(bContext *C, wmOperator *op)
   WM_operator_properties_border_to_rcti(op, &rect);
 
   rv3d->rflag |= RV3D_CLIPPING;
-  rv3d->clipbb = static_cast<BoundBox *>(MEM_callocN(sizeof(BoundBox), "clipbb"));
+  rv3d->clipbb = MEM_callocN<BoundBox>("clipbb");
 
   /* nullptr object because we don't want it in object space */
   ED_view3d_clipping_calc(rv3d->clipbb, rv3d->clip, region, nullptr, &rect);
@@ -786,7 +790,7 @@ static int view3d_clipping_exec(bContext *C, wmOperator *op)
   return OPERATOR_FINISHED;
 }
 
-static int view3d_clipping_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus view3d_clipping_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   RegionView3D *rv3d = CTX_wm_region_view3d(C);
   ARegion *region = CTX_wm_region(C);
@@ -808,7 +812,7 @@ void VIEW3D_OT_clip_border(wmOperatorType *ot)
   ot->description = "Set the view clipping region";
   ot->idname = "VIEW3D_OT_clip_border";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = view3d_clipping_invoke;
   ot->exec = view3d_clipping_exec;
   ot->modal = WM_gesture_box_modal;
@@ -903,7 +907,7 @@ void ED_view3d_cursor3d_position_rotation(bContext *C,
   }
   else if (orientation == V3D_CURSOR_ORIENT_XFORM) {
     float mat[3][3];
-    ED_transform_calc_orientation_from_type(C, mat);
+    blender::ed::transform::calc_orientation_from_type(C, mat);
     mat3_to_quat(r_cursor_quat, mat);
   }
   else if (orientation == V3D_CURSOR_ORIENT_GEOM) {
@@ -914,31 +918,33 @@ void ED_view3d_cursor3d_position_rotation(bContext *C,
     float ray_no[3];
     float ray_co[3];
 
-    SnapObjectContext *snap_context = ED_transform_snap_object_context_create(scene, 0);
+    blender::ed::transform::SnapObjectContext *snap_context =
+        blender::ed::transform::snap_object_context_create(scene, 0);
 
     float obmat[4][4];
     const Object *ob_dummy = nullptr;
     float dist_px = 0;
-    SnapObjectParams params{};
+    blender::ed::transform::SnapObjectParams params{};
     params.snap_target_select = SCE_SNAP_TARGET_ALL;
-    params.edit_mode_type = SNAP_GEOM_FINAL;
-    params.occlusion_test = SNAP_OCCLUSION_AS_SEEM;
-    if (ED_transform_snap_object_project_view3d_ex(snap_context,
-                                                   CTX_data_ensure_evaluated_depsgraph(C),
-                                                   region,
-                                                   v3d,
-                                                   SCE_SNAP_TO_FACE,
-                                                   &params,
-                                                   nullptr,
-                                                   mval_fl,
-                                                   nullptr,
-                                                   &dist_px,
-                                                   ray_co,
-                                                   ray_no,
-                                                   nullptr,
-                                                   &ob_dummy,
-                                                   obmat,
-                                                   nullptr) != 0)
+    params.edit_mode_type = blender::ed::transform::SNAP_GEOM_FINAL;
+    params.occlusion_test = blender::ed::transform::SNAP_OCCLUSION_AS_SEEM;
+    if (blender::ed::transform::snap_object_project_view3d_ex(
+            snap_context,
+            CTX_data_ensure_evaluated_depsgraph(C),
+            region,
+            v3d,
+            SCE_SNAP_TO_FACE,
+            &params,
+            nullptr,
+            mval_fl,
+            nullptr,
+            &dist_px,
+            ray_co,
+            ray_no,
+            nullptr,
+            &ob_dummy,
+            obmat,
+            nullptr) != 0)
     {
       if (use_depth) {
         copy_v3_v3(r_cursor_co, ray_co);
@@ -991,7 +997,7 @@ void ED_view3d_cursor3d_position_rotation(bContext *C,
         mul_qt_qtqt(r_cursor_quat, tquat_best, r_cursor_quat);
       }
     }
-    ED_transform_snap_object_context_destroy(snap_context);
+    blender::ed::transform::snap_object_context_destroy(snap_context);
   }
 }
 
@@ -1070,7 +1076,7 @@ void ED_view3d_cursor3d_update(bContext *C,
   DEG_id_tag_update(&scene->id, ID_RECALC_SYNC_TO_EVAL);
 }
 
-static int view3d_cursor3d_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus view3d_cursor3d_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
   bool use_depth = (U.uiflag & USER_DEPTH_CURSOR);
   {
@@ -1097,7 +1103,7 @@ void VIEW3D_OT_cursor3d(wmOperatorType *ot)
   ot->description = "Set the location of the 3D cursor";
   ot->idname = "VIEW3D_OT_cursor3d";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = view3d_cursor3d_invoke;
 
   ot->poll = ED_operator_region_view3d_active;
@@ -1145,7 +1151,7 @@ static const EnumPropertyItem prop_shading_type_items[] = {
     {0, nullptr, 0, nullptr, nullptr},
 };
 
-static int toggle_shading_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus toggle_shading_exec(bContext *C, wmOperator *op)
 {
   Main *bmain = CTX_data_main(C);
   View3D *v3d = CTX_wm_view3d(C);
@@ -1192,7 +1198,7 @@ void VIEW3D_OT_toggle_shading(wmOperatorType *ot)
   ot->description = "Toggle shading type in 3D viewport";
   ot->idname = "VIEW3D_OT_toggle_shading";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = toggle_shading_exec;
   ot->poll = ED_operator_view3d_active;
 
@@ -1207,7 +1213,7 @@ void VIEW3D_OT_toggle_shading(wmOperatorType *ot)
 /** \name Toggle XRay
  * \{ */
 
-static int toggle_xray_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus toggle_xray_exec(bContext *C, wmOperator *op)
 {
   View3D *v3d = CTX_wm_view3d(C);
   ScrArea *area = CTX_wm_area(C);
@@ -1246,7 +1252,7 @@ void VIEW3D_OT_toggle_xray(wmOperatorType *ot)
   ot->idname = "VIEW3D_OT_toggle_xray";
   ot->description = "Transparent scene display. Allow selecting through items";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = toggle_xray_exec;
   ot->poll = ED_operator_view3d_active;
 }
