@@ -12,20 +12,15 @@
 #include "RNA_define.hh"
 #include "RNA_enum_types.hh"
 
-#include "rna_internal.h"
+#include "rna_internal.hh"
 
 #include "DNA_action_types.h"
-#include "DNA_armature_types.h"
-#include "DNA_constraint_types.h"
 #include "DNA_object_types.h"
-#include "DNA_scene_types.h"
 
-#include "BLI_math_rotation.h"
-#include "BLI_math_vector.h"
+#include "BLI_math_base.h"
+
 #include "BLI_string_utf8_symbols.h"
 
-#include "BLT_translation.h"
-
 #include "UI_resources.hh"
 
 #include "WM_types.hh"
@@ -59,21 +54,28 @@ const EnumPropertyItem rna_enum_color_sets_items[] = {
 
 #ifdef RNA_RUNTIME
 
-#  include "BLI_ghash.h"
-#  include "BLI_string_utils.h"
+#  include <algorithm>
 
-#  include "BIK_api.h"
-#  include "BKE_action.h"
-#  include "BKE_armature.h"
+#  include <fmt/format.h>
 
-#  include "DNA_userdef_types.h"
+#  include "BLI_math_vector.h"
+#  include "BLI_string.h"
+#  include "BLI_string_utf8.h"
 
 #  include "MEM_guardedalloc.h"
 
+#  include "DNA_constraint_types.h"
+
+#  include "BIK_api.h"
+
+#  include "BKE_action.hh"
+#  include "BKE_armature.hh"
 #  include "BKE_constraint.h"
-#  include "BKE_context.h"
-#  include "BKE_global.h"
-#  include "BKE_idprop.h"
+#  include "BKE_context.hh"
+#  include "BKE_global.hh"
+#  include "BKE_lib_id.hh"
+#  include "BKE_main.hh"
+#  include "BKE_report.hh"
 
 #  include "DEG_depsgraph.hh"
 #  include "DEG_depsgraph_build.hh"
@@ -103,7 +105,7 @@ static void rna_Pose_dependency_update(Main *bmain, Scene * /*scene*/, PointerRN
 
 static void rna_Pose_IK_update(Main * /*bmain*/, Scene * /*scene*/, PointerRNA *ptr)
 {
-  // ob->pose->flag |= (POSE_LOCKED | POSE_DO_UNLOCK); /* XXX: when to use this?  */
+  // ob->pose->flag |= (POSE_LOCKED | POSE_DO_UNLOCK); /* XXX: when to use this? */
   Object *ob = (Object *)ptr->owner_id;
 
   DEG_id_tag_update(&ob->id, ID_RECALC_GEOMETRY);
@@ -112,18 +114,18 @@ static void rna_Pose_IK_update(Main * /*bmain*/, Scene * /*scene*/, PointerRNA *
   BIK_clear_data(ob->pose);
 }
 
-static char *rna_Pose_path(const PointerRNA * /*ptr*/)
+static std::optional<std::string> rna_Pose_path(const PointerRNA * /*ptr*/)
 {
-  return BLI_strdup("pose");
+  return "pose";
 }
 
-static char *rna_PoseBone_path(const PointerRNA *ptr)
+static std::optional<std::string> rna_PoseBone_path(const PointerRNA *ptr)
 {
   const bPoseChannel *pchan = static_cast<const bPoseChannel *>(ptr->data);
   char name_esc[sizeof(pchan->name) * 2];
 
   BLI_str_escape(name_esc, pchan->name, sizeof(name_esc));
-  return BLI_sprintfN("pose.bones[\"%s\"]", name_esc);
+  return fmt::format("pose.bones[\"{}\"]", name_esc);
 }
 
 /* shared for actions groups and bone groups */
@@ -195,7 +197,7 @@ static void rna_Pose_ik_solver_update(Main *bmain, Scene * /*scene*/, PointerRNA
 
   BKE_pose_update_constraint_flags(pose);
 
-  object_test_constraints(bmain, ob);
+  blender::ed::object::object_test_constraints(bmain, ob);
 
   DEG_id_tag_update(&ob->id, ID_RECALC_GEOMETRY | ID_RECALC_TRANSFORM);
 }
@@ -276,7 +278,7 @@ static PointerRNA rna_PoseChannel_bone_get(PointerRNA *ptr)
   /* Replace the id_data pointer with the Armature ID. */
   tmp_ptr.owner_id = static_cast<ID *>(ob->data);
 
-  return rna_pointer_inherit_refine(&tmp_ptr, &RNA_Bone, pchan->bone);
+  return RNA_pointer_create_with_parent(tmp_ptr, &RNA_Bone, pchan->bone);
 }
 
 static bool rna_PoseChannel_has_ik_get(PointerRNA *ptr)
@@ -302,7 +304,7 @@ static StructRNA *rna_IKParam_refine(PointerRNA *ptr)
 static PointerRNA rna_Pose_ikparam_get(PointerRNA *ptr)
 {
   bPose *pose = (bPose *)ptr->data;
-  return rna_pointer_inherit_refine(ptr, &RNA_IKParam, pose->ikparam);
+  return RNA_pointer_create_with_parent(*ptr, &RNA_IKParam, pose->ikparam);
 }
 
 static StructRNA *rna_Pose_ikparam_typef(PointerRNA *ptr)
@@ -323,27 +325,13 @@ static void rna_Itasc_update(Main * /*bmain*/, Scene * /*scene*/, PointerRNA *pt
   bItasc *itasc = static_cast<bItasc *>(ptr->data);
 
   /* verify values */
-  if (itasc->precision < 0.0001f) {
-    itasc->precision = 0.0001f;
-  }
-  if (itasc->minstep < 0.001f) {
-    itasc->minstep = 0.001f;
-  }
-  if (itasc->maxstep < itasc->minstep) {
-    itasc->maxstep = itasc->minstep;
-  }
-  if (itasc->feedback < 0.01f) {
-    itasc->feedback = 0.01f;
-  }
-  if (itasc->feedback > 100.0f) {
-    itasc->feedback = 100.0f;
-  }
-  if (itasc->maxvel < 0.01f) {
-    itasc->maxvel = 0.01f;
-  }
-  if (itasc->maxvel > 100.0f) {
-    itasc->maxvel = 100.0f;
-  }
+  itasc->precision = std::max(itasc->precision, 0.0001f);
+  itasc->minstep = std::max(itasc->minstep, 0.001f);
+  itasc->maxstep = std::max(itasc->maxstep, itasc->minstep);
+  itasc->feedback = std::max(itasc->feedback, 0.01f);
+  itasc->feedback = std::min(itasc->feedback, 100.0f);
+  itasc->maxvel = std::max(itasc->maxvel, 0.01f);
+  itasc->maxvel = std::min(itasc->maxvel, 100.0f);
   BIK_update_param(ob->pose);
 
   DEG_id_tag_update(&ob->id, ID_RECALC_GEOMETRY);
@@ -362,7 +350,7 @@ static PointerRNA rna_PoseChannel_active_constraint_get(PointerRNA *ptr)
 {
   bPoseChannel *pchan = (bPoseChannel *)ptr->data;
   bConstraint *con = BKE_constraints_active_get(&pchan->constraints);
-  return rna_pointer_inherit_refine(ptr, &RNA_Constraint, con);
+  return RNA_pointer_create_with_parent(*ptr, &RNA_Constraint, con);
 }
 
 static void rna_PoseChannel_active_constraint_set(PointerRNA *ptr,
@@ -381,7 +369,7 @@ static bConstraint *rna_PoseChannel_constraints_new(ID *id,
   Object *ob = (Object *)id;
   bConstraint *new_con = BKE_constraint_add_for_pose(ob, pchan, nullptr, type);
 
-  ED_object_constraint_dependency_tag_update(main, ob, new_con);
+  blender::ed::object::constraint_dependency_tag_update(main, ob, new_con);
   WM_main_add_notifier(NC_OBJECT | ND_CONSTRAINT | NA_ADDED, id);
 
   return new_con;
@@ -400,10 +388,11 @@ static void rna_PoseChannel_constraints_remove(
     return;
   }
 
-  BKE_constraint_remove(&pchan->constraints, con);
-  RNA_POINTER_INVALIDATE(con_ptr);
+  BKE_constraint_remove_ex(&pchan->constraints, ob, con);
+  con_ptr->invalidate();
 
-  ED_object_constraint_update(bmain, ob);
+  blender::ed::object::constraint_update(bmain, ob);
+  DEG_relations_tag_update(bmain);
 
   /* XXX(@ideasman42): is this really needed? */
   BKE_constraints_active_set(&pchan->constraints, nullptr);
@@ -429,7 +418,7 @@ static void rna_PoseChannel_constraints_move(
     return;
   }
 
-  ED_object_constraint_tag_update(bmain, ob, nullptr);
+  blender::ed::object::constraint_tag_update(bmain, ob, nullptr);
   WM_main_add_notifier(NC_OBJECT | ND_CONSTRAINT, ob);
 }
 
@@ -443,7 +432,7 @@ static bConstraint *rna_PoseChannel_constraints_copy(ID *id,
   bConstraint *new_con = BKE_constraint_copy_for_pose(ob, pchan, con);
   new_con->flag |= CONSTRAINT_OVERRIDE_LIBRARY_LOCAL;
 
-  ED_object_constraint_dependency_tag_update(bmain, ob, new_con);
+  blender::ed::object::constraint_dependency_tag_update(bmain, ob, new_con);
   WM_main_add_notifier(NC_OBJECT | ND_CONSTRAINT | NA_ADDED, id);
 
   return new_con;
@@ -457,8 +446,8 @@ bool rna_PoseChannel_constraints_override_apply(Main *bmain,
   PropertyRNA *prop_dst = rnaapply_ctx.prop_dst;
   IDOverrideLibraryPropertyOperation *opop = rnaapply_ctx.liboverride_operation;
 
-  BLI_assert(opop->operation == LIBOVERRIDE_OP_INSERT_AFTER &&
-             "Unsupported RNA override operation on constraints collection");
+  BLI_assert_msg(opop->operation == LIBOVERRIDE_OP_INSERT_AFTER,
+                 "Unsupported RNA override operation on constraints collection");
 
   bPoseChannel *pchan_dst = (bPoseChannel *)ptr_dst->data;
   bPoseChannel *pchan_src = (bPoseChannel *)ptr_src->data;
@@ -497,7 +486,7 @@ bool rna_PoseChannel_constraints_override_apply(Main *bmain,
   return true;
 }
 
-static int rna_PoseChannel_proxy_editable(PointerRNA * /*ptr*/, const char ** /*r_info*/)
+static int rna_PoseChannel_proxy_editable(const PointerRNA * /*ptr*/, const char ** /*r_info*/)
 {
 #  if 0
   Object *ob = (Object *)ptr->owner_id;
@@ -513,7 +502,7 @@ static int rna_PoseChannel_proxy_editable(PointerRNA * /*ptr*/, const char ** /*
   return PROP_EDITABLE;
 }
 
-static int rna_PoseChannel_location_editable(PointerRNA *ptr, int index)
+static int rna_PoseChannel_location_editable(const PointerRNA *ptr, int index)
 {
   bPoseChannel *pchan = (bPoseChannel *)ptr->data;
 
@@ -521,18 +510,16 @@ static int rna_PoseChannel_location_editable(PointerRNA *ptr, int index)
   if ((index == 0) && (pchan->protectflag & OB_LOCK_LOCX)) {
     return 0;
   }
-  else if ((index == 1) && (pchan->protectflag & OB_LOCK_LOCY)) {
+  if ((index == 1) && (pchan->protectflag & OB_LOCK_LOCY)) {
     return 0;
   }
-  else if ((index == 2) && (pchan->protectflag & OB_LOCK_LOCZ)) {
+  if ((index == 2) && (pchan->protectflag & OB_LOCK_LOCZ)) {
     return 0;
   }
-  else {
-    return PROP_EDITABLE;
-  }
+  return PROP_EDITABLE;
 }
 
-static int rna_PoseChannel_scale_editable(PointerRNA *ptr, int index)
+static int rna_PoseChannel_scale_editable(const PointerRNA *ptr, int index)
 {
   bPoseChannel *pchan = (bPoseChannel *)ptr->data;
 
@@ -540,18 +527,16 @@ static int rna_PoseChannel_scale_editable(PointerRNA *ptr, int index)
   if ((index == 0) && (pchan->protectflag & OB_LOCK_SCALEX)) {
     return 0;
   }
-  else if ((index == 1) && (pchan->protectflag & OB_LOCK_SCALEY)) {
+  if ((index == 1) && (pchan->protectflag & OB_LOCK_SCALEY)) {
     return 0;
   }
-  else if ((index == 2) && (pchan->protectflag & OB_LOCK_SCALEZ)) {
+  if ((index == 2) && (pchan->protectflag & OB_LOCK_SCALEZ)) {
     return 0;
   }
-  else {
-    return PROP_EDITABLE;
-  }
+  return PROP_EDITABLE;
 }
 
-static int rna_PoseChannel_rotation_euler_editable(PointerRNA *ptr, int index)
+static int rna_PoseChannel_rotation_euler_editable(const PointerRNA *ptr, int index)
 {
   bPoseChannel *pchan = (bPoseChannel *)ptr->data;
 
@@ -559,18 +544,16 @@ static int rna_PoseChannel_rotation_euler_editable(PointerRNA *ptr, int index)
   if ((index == 0) && (pchan->protectflag & OB_LOCK_ROTX)) {
     return 0;
   }
-  else if ((index == 1) && (pchan->protectflag & OB_LOCK_ROTY)) {
+  if ((index == 1) && (pchan->protectflag & OB_LOCK_ROTY)) {
     return 0;
   }
-  else if ((index == 2) && (pchan->protectflag & OB_LOCK_ROTZ)) {
+  if ((index == 2) && (pchan->protectflag & OB_LOCK_ROTZ)) {
     return 0;
   }
-  else {
-    return PROP_EDITABLE;
-  }
+  return PROP_EDITABLE;
 }
 
-static int rna_PoseChannel_rotation_4d_editable(PointerRNA *ptr, int index)
+static int rna_PoseChannel_rotation_4d_editable(const PointerRNA *ptr, int index)
 {
   bPoseChannel *pchan = (bPoseChannel *)ptr->data;
 
@@ -580,13 +563,13 @@ static int rna_PoseChannel_rotation_4d_editable(PointerRNA *ptr, int index)
     if ((index == 0) && (pchan->protectflag & OB_LOCK_ROTW)) {
       return 0;
     }
-    else if ((index == 1) && (pchan->protectflag & OB_LOCK_ROTX)) {
+    if ((index == 1) && (pchan->protectflag & OB_LOCK_ROTX)) {
       return 0;
     }
-    else if ((index == 2) && (pchan->protectflag & OB_LOCK_ROTY)) {
+    if ((index == 2) && (pchan->protectflag & OB_LOCK_ROTY)) {
       return 0;
     }
-    else if ((index == 3) && (pchan->protectflag & OB_LOCK_ROTZ)) {
+    if ((index == 3) && (pchan->protectflag & OB_LOCK_ROTZ)) {
       return 0;
     }
   }
@@ -595,17 +578,15 @@ static int rna_PoseChannel_rotation_4d_editable(PointerRNA *ptr, int index)
 }
 
 /* not essential, but much faster than the default lookup function */
-static int rna_PoseBones_lookup_string(PointerRNA *ptr, const char *key, PointerRNA *r_ptr)
+static bool rna_PoseBones_lookup_string(PointerRNA *ptr, const char *key, PointerRNA *r_ptr)
 {
   bPose *pose = (bPose *)ptr->data;
   bPoseChannel *pchan = BKE_pose_channel_find_name(pose, key);
   if (pchan) {
-    *r_ptr = RNA_pointer_create(ptr->owner_id, &RNA_PoseBone, pchan);
+    rna_pointer_create_with_ancestors(*ptr, &RNA_PoseBone, pchan, *r_ptr);
     return true;
   }
-  else {
-    return false;
-  }
+  return false;
 }
 
 static void rna_PoseChannel_matrix_basis_get(PointerRNA *ptr, float *values)
@@ -654,8 +635,37 @@ static void rna_PoseChannel_custom_shape_transform_set(PointerRNA *ptr,
   bPoseChannel *pchan = (bPoseChannel *)ptr->data;
   Object *ob = (Object *)ptr->owner_id;
 
-  pchan->custom_tx = static_cast<bPoseChannel *>(rna_PoseChannel_ensure_own_pchan(
-      ob, (Object *)value.owner_id, static_cast<bPoseChannel *>(value.data)));
+  pchan->custom_tx = rna_PoseChannel_ensure_own_pchan(
+      ob, (Object *)value.owner_id, static_cast<bPoseChannel *>(value.data));
+}
+
+void rna_Pose_custom_shape_set(PointerRNA *ptr, PointerRNA value, struct ReportList *reports)
+{
+  bPoseChannel *pchan = static_cast<bPoseChannel *>(ptr->data);
+  Object *custom_shape = static_cast<Object *>(value.data);
+
+  if (!custom_shape) {
+    id_us_min(reinterpret_cast<ID *>(pchan->custom));
+    pchan->custom = nullptr;
+    return;
+  }
+
+  /* This should be ensured by the RNA property type. */
+  BLI_assert(GS(custom_shape->id.name) == ID_OB);
+
+  if (custom_shape->type == OB_ARMATURE) {
+    BKE_report(reports, RPT_ERROR, "Cannot use armature object as custom bone shape");
+    return;
+  }
+
+  id_us_min(reinterpret_cast<ID *>(pchan->custom));
+  pchan->custom = custom_shape;
+  id_us_plus(reinterpret_cast<ID *>(pchan->custom));
+}
+
+bool rna_Pose_custom_shape_object_poll(PointerRNA * /*ptr*/, PointerRNA value)
+{
+  return (reinterpret_cast<Object *>(value.owner_id))->type != OB_ARMATURE;
 }
 
 #else
@@ -837,7 +847,6 @@ static void rna_def_pose_channel(BlenderRNA *brna)
   RNA_def_property_update(prop, NC_OBJECT | ND_POSE, "rna_Pose_IK_update");
 
   prop = RNA_def_property(srna, "scale", PROP_FLOAT, PROP_XYZ);
-  RNA_def_property_float_sdna(prop, nullptr, "size");
   RNA_def_property_flag(prop, PROP_PROPORTIONAL);
   RNA_def_property_override_flag(prop, PROPOVERRIDE_OVERRIDABLE_LIBRARY);
   RNA_def_property_editable_array_func(prop, "rna_PoseChannel_scale_editable");
@@ -885,7 +894,11 @@ static void rna_def_pose_channel(BlenderRNA *brna)
   /* XXX... disabled, since proxy-locked layers are currently
    * used for ensuring proxy-syncing too */
   RNA_def_property_editable_func(prop, "rna_PoseChannel_proxy_editable");
-  RNA_def_property_ui_text(prop, "Rotation Mode", "");
+  RNA_def_property_ui_text(
+      prop,
+      "Rotation Mode",
+      /* This description is shared by other "rotation_mode" properties. */
+      "The kind of rotation to apply, values from other rotation modes aren't used");
   RNA_def_property_update(prop, NC_OBJECT | ND_POSE, "rna_Pose_update");
 
   /* Curved bones settings - Applied on top of rest-pose values. */
@@ -967,6 +980,7 @@ static void rna_def_pose_channel(BlenderRNA *brna)
   prop = RNA_def_property(srna, "is_in_ik_chain", PROP_BOOLEAN, PROP_NONE);
   RNA_def_property_boolean_funcs(prop, "rna_PoseChannel_has_ik_get", nullptr);
   RNA_def_property_clear_flag(prop, PROP_EDITABLE);
+  RNA_def_property_override_flag(prop, PROPOVERRIDE_NO_COMPARISON);
   RNA_def_property_ui_text(prop, "Has IK", "Is part of an IK chain");
   RNA_def_property_update(prop, NC_OBJECT | ND_POSE, "rna_Pose_IK_update");
 
@@ -1115,6 +1129,8 @@ static void rna_def_pose_channel(BlenderRNA *brna)
   RNA_def_property_ui_text(
       prop, "Custom Object", "Object that defines custom display shape for this bone");
   RNA_def_property_editable_func(prop, "rna_PoseChannel_proxy_editable");
+  RNA_def_property_pointer_funcs(
+      prop, nullptr, "rna_Pose_custom_shape_set", nullptr, "rna_Pose_custom_shape_object_poll");
   RNA_def_property_update(prop, NC_OBJECT | ND_POSE, "rna_Pose_dependency_update");
 
   prop = RNA_def_property(srna, "custom_shape_scale_xyz", PROP_FLOAT, PROP_XYZ);
@@ -1159,6 +1175,15 @@ static void rna_def_pose_channel(BlenderRNA *brna)
       prop, nullptr, "rna_PoseChannel_custom_shape_transform_set", nullptr, nullptr);
   RNA_def_property_update(prop, NC_OBJECT | ND_POSE, "rna_Pose_update");
 
+  prop = RNA_def_property(srna, "custom_shape_wire_width", PROP_FLOAT, PROP_NONE);
+  RNA_def_property_float_sdna(prop, nullptr, "custom_shape_wire_width");
+  RNA_def_property_ui_text(prop, "Wire Width", "Adjust the line thickness of custom shapes");
+  /* When changing the upper limit of the range, also adjust the WIRE_WIDTH_COMPRESSION in
+   * overlay_shader_shared.hh */
+  RNA_def_property_range(prop, 1.0f, 16.0f);
+  RNA_def_property_ui_range(prop, 1.0f, 10.0f, 1, 1);
+  RNA_def_property_update(prop, NC_OBJECT | ND_POSE, "rna_Pose_update");
+
   prop = RNA_def_property(srna, "color", PROP_POINTER, PROP_NONE);
   RNA_def_property_struct_type(prop, "BoneColor");
   RNA_def_property_clear_flag(prop, PROP_EDITABLE);
@@ -1166,18 +1191,18 @@ static void rna_def_pose_channel(BlenderRNA *brna)
 
   /* transform locks */
   prop = RNA_def_property(srna, "lock_location", PROP_BOOLEAN, PROP_NONE);
-  RNA_def_property_boolean_sdna(prop, nullptr, "protectflag", OB_LOCK_LOCX);
-  RNA_def_property_array(prop, 3);
+  RNA_def_property_boolean_bitset_array_sdna(prop, nullptr, "protectflag", OB_LOCK_LOCX, 3);
   RNA_def_property_ui_text(prop, "Lock Location", "Lock editing of location when transforming");
   RNA_def_property_ui_icon(prop, ICON_UNLOCKED, 1);
+  RNA_def_property_override_flag(prop, PROPOVERRIDE_OVERRIDABLE_LIBRARY);
   RNA_def_property_editable_func(prop, "rna_PoseChannel_proxy_editable");
   RNA_def_property_update(prop, NC_OBJECT | ND_POSE, "rna_Pose_update");
 
   prop = RNA_def_property(srna, "lock_rotation", PROP_BOOLEAN, PROP_NONE);
-  RNA_def_property_boolean_sdna(prop, nullptr, "protectflag", OB_LOCK_ROTX);
-  RNA_def_property_array(prop, 3);
+  RNA_def_property_boolean_bitset_array_sdna(prop, nullptr, "protectflag", OB_LOCK_ROTX, 3);
   RNA_def_property_ui_text(prop, "Lock Rotation", "Lock editing of rotation when transforming");
   RNA_def_property_ui_icon(prop, ICON_UNLOCKED, 1);
+  RNA_def_property_override_flag(prop, PROPOVERRIDE_OVERRIDABLE_LIBRARY);
   RNA_def_property_editable_func(prop, "rna_PoseChannel_proxy_editable");
   RNA_def_property_update(prop, NC_OBJECT | ND_POSE, "rna_Pose_update");
 
@@ -1190,6 +1215,7 @@ static void rna_def_pose_channel(BlenderRNA *brna)
       "Lock Rotation (4D Angle)",
       "Lock editing of 'angle' component of four-component rotations when transforming");
   RNA_def_property_ui_icon(prop, ICON_UNLOCKED, 1);
+  RNA_def_property_override_flag(prop, PROPOVERRIDE_OVERRIDABLE_LIBRARY);
   RNA_def_property_editable_func(prop, "rna_PoseChannel_proxy_editable");
   RNA_def_property_update(prop, NC_OBJECT | ND_POSE, "rna_Pose_update");
 
@@ -1200,14 +1226,15 @@ static void rna_def_pose_channel(BlenderRNA *brna)
       prop,
       "Lock Rotations (4D)",
       "Lock editing of four component rotations by components (instead of as Eulers)");
+  RNA_def_property_override_flag(prop, PROPOVERRIDE_OVERRIDABLE_LIBRARY);
   RNA_def_property_editable_func(prop, "rna_PoseChannel_proxy_editable");
   RNA_def_property_update(prop, NC_OBJECT | ND_POSE, "rna_Pose_update");
 
   prop = RNA_def_property(srna, "lock_scale", PROP_BOOLEAN, PROP_NONE);
-  RNA_def_property_boolean_sdna(prop, nullptr, "protectflag", OB_LOCK_SCALEX);
-  RNA_def_property_array(prop, 3);
+  RNA_def_property_boolean_bitset_array_sdna(prop, nullptr, "protectflag", OB_LOCK_SCALEX, 3);
   RNA_def_property_ui_text(prop, "Lock Scale", "Lock editing of scale when transforming");
   RNA_def_property_ui_icon(prop, ICON_UNLOCKED, 1);
+  RNA_def_property_override_flag(prop, PROPOVERRIDE_OVERRIDABLE_LIBRARY);
   RNA_def_property_editable_func(prop, "rna_PoseChannel_proxy_editable");
   RNA_def_property_update(prop, NC_OBJECT | ND_POSE, "rna_Pose_update");
 
